@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "atom.h"
 
 #include "bond.h"
@@ -296,7 +297,8 @@ atom::atom(element p1, const fGL * p2, i32u p3)
 	varind = NOT_DEFINED;
 	
 	flags = 0;
-	ecomp_grp_i = NOT_DEFINED;
+	
+	ecomp_grp_i = 0;	// the "default" group index is zero!
 }
 
 atom::atom(const atom & p1)
@@ -336,6 +338,7 @@ atom::atom(const atom & p1)
 	varind = p1.varind;
 	
 	flags = p1.flags;
+	
 	ecomp_grp_i = p1.ecomp_grp_i;
 }
 
