@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "eng1_mm.h"
 
 #include "eng1_mm_tripos52.h"
@@ -17,6 +18,10 @@
 #include <sstream>
 using namespace std;
 
+#ifdef WIN32
+#include "win32_hacks.h"
+#endif	// WIN32
+
 /*################################################################################################*/
 
 const i32u setup1_mm::eng_id_tab[] =
@@ -123,7 +128,19 @@ engine * setup1_mm::CreateEngineByIndex(i32u eng_index)
 prmfit_tables * tab = NULL; eng1_mm_prmfit * eng = NULL;
 
 ostringstream fps;
-fps << model::libdata_path << "/param_mm/prmfit" << ends;
+#ifndef WIN32
+fps << model::libdata_path << DIR_SEPARATOR;
+fps << model::libversion << DIR_SEPARATOR;
+fps << "param_mm/prmfit" << ends;
+#else	// WIN32
+const char * libdata_path = get_libdata_path_from_registry();
+
+fps << libdata_path << DIR_SEPARATOR;
+fps << "libghemical" << DIR_SEPARATOR;
+fps << model::libversion << DIR_SEPARATOR;
+fps << "param_mm\prmfit" << ends;
+#endif	// WIN32
+
 char forcefield_path[256]; strcpy(forcefield_path, fps.str().c_str());
 	
 	switch (eng_id_tab[eng_index])
@@ -141,7 +158,7 @@ if (GetModel()->use_boundary_potential) GetModel()->Message("use_boundary_potent
 		case ENG1_MM_PERIODIC:
 		GetModel()->use_periodic_boundary_conditions = true;
 	if (!GetModel()->IsGroupsClean()) GetModel()->UpdateGroups();
-	if (!GetModel()->IsGroupsSorted()) GetModel()->SortGroups(false);
+	if (!GetModel()->IsGroupsSorted()) GetModel()->SortGroups();
 		return new eng1_mm_default_mim(this, 1);
 		
 		case ENG1_MM_EXPERIMENTAL:
@@ -269,10 +286,6 @@ void eng1_mm::Compute(i32u p1, bool dv_flag)
 	virial[0] = virial[1] = virial[2] = 0.0;
 	do_virial = dv_flag;
 	
-	E_solute = 0.0;		// re-think this for eng2_xx_yy classes!!!
-	E_solvent = 0.0;	// re-think this for eng2_xx_yy classes!!!
-	E_solusolv = 0.0;	// re-think this for eng2_xx_yy classes!!!
-	
 	if (p1)
 	{
 		for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
@@ -283,6 +296,8 @@ void eng1_mm::Compute(i32u p1, bool dv_flag)
 		}
 	}
 	
+	if (ECOMPstore != NULL) ecomp_AddCycle();
+	
 	ComputeBT1(p1);
 	
 	// communicate via MPI if parallel?!?!?!
@@ -309,7 +324,7 @@ void eng1_mm::Compute(i32u p1, bool dv_flag)
 
 void eng1_mm::SearchCR1a(atom * atm1)
 {
-	for (iter_cl itc = atm1->cr_list.begin();itc != atm1->cr_list.end();itc++)
+	for (iter_cl itc = atm1->GetConnRecsBegin();itc != atm1->GetConnRecsEnd();itc++)
 	{
 		atom * atm2 = (* itc).atmr;
 		cr1.push_back(atm2);
@@ -320,7 +335,7 @@ void eng1_mm::SearchCR1a(atom * atm1)
 
 void eng1_mm::SearchCR1b(atom * atm2, bond * reserved1)
 {
-	for (iter_cl itc = atm2->cr_list.begin();itc != atm2->cr_list.end();itc++)
+	for (iter_cl itc = atm2->GetConnRecsBegin();itc != atm2->GetConnRecsEnd();itc++)
 	{
 		if ((* itc).bndr == reserved1) continue;
 		
@@ -333,7 +348,7 @@ void eng1_mm::SearchCR1b(atom * atm2, bond * reserved1)
 
 void eng1_mm::SearchCR2(atom * atm3, bond * reserved1, bond * reserved2)
 {
-	for (iter_cl itc = atm3->cr_list.begin();itc != atm3->cr_list.end();itc++)
+	for (iter_cl itc = atm3->GetConnRecsBegin();itc != atm3->GetConnRecsEnd();itc++)
 	{
 		if ((* itc).bndr == reserved1) continue;
 		if ((* itc).bndr == reserved2) continue;
