@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "eng1_mm_default.h"
 
 #include "v3d.h"
@@ -43,7 +44,8 @@ eng1_mm_default_bt::eng1_mm_default_bt(setup * p1, i32u p2) :
 	
 	atom ** glob_atmtab = GetSetup()->GetAtoms();
 	
-	ostream * ostr = default_tables::GetInstance()->ostr;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 /*##############################################*/
 /*##############################################*/
@@ -96,14 +98,15 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 			
 			success = default_tables::GetInstance()->e_Init(this, & newbt1, bt);
 		}
-		else
+		
+		if (success != true)
 		{
 			default_bs_query query; query.strict = false;
 			query.atmtp[0] = atmtab[newbt1.atmi[0]]->atmtp;
 			query.atmtp[1] = atmtab[newbt1.atmi[1]]->atmtp;
 			query.bndtp = bndtab[n1]->bt.GetValue();
 			
-			default_tables::GetInstance()->DoParamSearch(& query);
+			default_tables::GetInstance()->DoParamSearch(& query, GetSetup()->GetModel());
 			if (query.index != NOT_DEFINED) success = true;
 			
 			newbt1.opt = query.opt;
@@ -134,7 +137,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	
 	for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 	{
-		if (atmtab[n1]->cr_list.size() < 2) continue;
+		if (atmtab[n1]->GetConnRecCount() < 2) continue;
 		
 		// central atom is known, now find all possible combinations of bonds...
 		
@@ -142,9 +145,9 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 		iter_cl ita; iter_cl rnga[2];
 		iter_cl itb; iter_cl rngb[2];
 		
-		rngb[1] = atmtab[n1]->cr_list.end();
+		rngb[1] = atmtab[n1]->GetConnRecsEnd();
 		
-		rnga[0] = atmtab[n1]->cr_list.begin();
+		rnga[0] = atmtab[n1]->GetConnRecsBegin();
 		rnga[1] = rngb[1]; rnga[1]--;
 		
 		for (ita = rnga[0];ita != rnga[1];ita++)
@@ -174,7 +177,8 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 					
 					success = default_tables::GetInstance()->e_Init(this, & newbt2, bt);
 				}
-				else
+				
+				if (success != true)
 				{
 					default_ab_query query; query.strict = false;
 					query.atmtp[0] = atmtab[newbt2.atmi[0]]->atmtp;
@@ -183,7 +187,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 					query.bndtp[0] = (* ita).bndr->bt.GetValue();
 					query.bndtp[1] = (* itb).bndr->bt.GetValue();
 					
-					default_tables::GetInstance()->DoParamSearch(& query);
+					default_tables::GetInstance()->DoParamSearch(& query, GetSetup()->GetModel());
 					if (query.index != NOT_DEFINED) success = true;
 					
 					newbt2.opt = query.opt;
@@ -210,7 +214,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	
 	for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 	{
-		if (atmtab[n1]->cr_list.size() < 2) continue;
+		if (atmtab[n1]->GetConnRecCount() < 2) continue;
 		
 		// skip if default geometry is marked linear for this atomtype...
 		// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -218,12 +222,12 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	//	if (tp1 == NULL) { cout << "GetAtomType() failed! " << atmtab[n1]->atmtp << endl; exit(EXIT_FAILURE); }
 		if (tp1 != NULL && (tp1->flags & 3) == 1) continue;	// linear geometry -> undefined torsion.
 		
-		for (iter_cl it2 = atmtab[n1]->cr_list.begin();it2 != atmtab[n1]->cr_list.end();it2++)
+		for (iter_cl it2 = atmtab[n1]->GetConnRecsBegin();it2 != atmtab[n1]->GetConnRecsEnd();it2++)
 		{
 			bool another = (atmtab[n1] == (* it2).bndr->atmr[0]);
 			atom * atmr = (* it2).bndr->atmr[another];
 			
-			if (atmr->cr_list.size() < 2) continue;			// no torsion...
+			if (atmr->GetConnRecCount() < 2) continue;			// no torsion...
 			if (atmr->varind > atmtab[n1]->varind) continue;	// skip duplicates...
 			
 			// skip if default geometry is marked linear for this atomtype...
@@ -235,7 +239,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 			// central atoms are known, now find all possible combinations of bonds...
 			
 			vector<i32s> ind1a; vector<bool> dir1a;		// search for the 1st group...
-			for (iter_cl it3 = atmtab[n1]->cr_list.begin();it3 != atmtab[n1]->cr_list.end();it3++)
+			for (iter_cl it3 = atmtab[n1]->GetConnRecsBegin();it3 != atmtab[n1]->GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -263,7 +267,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 			}
 			
 			vector<i32s> ind1b; vector<bool> dir1b;		// search for the 2nd group...
-			for (iter_cl it3 = atmr->cr_list.begin();it3 != atmr->cr_list.end();it3++)
+			for (iter_cl it3 = atmr->GetConnRecsBegin();it3 != atmr->GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -340,7 +344,8 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 						
 						success = default_tables::GetInstance()->e_Init(this, & newbt3, bt);
 					}
-					else
+					
+					if (success != true)
 					{
 						default_tr_query query; query.strict = false;
 						query.atmtp[0] = atmtab[newbt3.atmi[0]]->atmtp;
@@ -352,7 +357,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 						query.bndtp[1] = (* itb2).bt.GetValue();
 						query.bndtp[2] = (* itb3).bt.GetValue();
 						
-						default_tables::GetInstance()->DoParamSearch(& query);
+						default_tables::GetInstance()->DoParamSearch(& query, GetSetup()->GetModel());
 						if (query.index != NOT_DEFINED) success = true;
 						
 						newbt3.fc1 = query.fc1;
@@ -385,10 +390,10 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 		const default_at * tp = default_tables::GetInstance()->GetAtomType(atmtab[n1]->atmtp);
 		if (!tp || (tp != NULL && !(tp->flags & 16))) continue;
 		
-		if (atmtab[n1]->cr_list.size() != 3) continue;
+		if (atmtab[n1]->GetConnRecCount() != 3) continue;
 		
 		crec * crtab[3];
-		iter_cl iter = atmtab[n1]->cr_list.begin();
+		iter_cl iter = atmtab[n1]->GetConnRecsBegin();
 		crtab[0] = & (* iter); iter++;
 		crtab[1] = & (* iter); iter++;
 		crtab[2] = & (* iter);
@@ -471,7 +476,8 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 				
 				success = default_tables::GetInstance()->e_Init(this, & newbt4, bt);
 			}
-			else
+			
+			if (success != true)
 			{
 				default_op_query query; query.strict = false;
 				query.atmtp[0] = atmtab[newbt4.atmi[0]]->atmtp;
@@ -483,7 +489,7 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 				query.bndtp[1] = crtab[(n2 + 1) % 3]->bndr->bt.GetValue();
 				query.bndtp[2] = crtab[(n2 + 2) % 3]->bndr->bt.GetValue();
 				
-				default_tables::GetInstance()->DoParamSearch(& query);
+				default_tables::GetInstance()->DoParamSearch(& query, GetSetup()->GetModel());
 				if (query.index != NOT_DEFINED) success = true;
 				
 				newbt4.opt = query.opt;
@@ -503,11 +509,11 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	// report possible errors...
 	
 	i32s total_err = bt1_err + bt2_err + bt3_err + bt4_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
+		GetSetup()->GetModel()->PrintToLog(str.str().c_str());
 	}
 }
 
@@ -565,8 +571,10 @@ i32s eng1_mm_default_bt::FindTorsion(atom * a1, atom * a2, atom * a3, atom * a4)
 	return NOT_DEFINED;
 }
 
-bool eng1_mm_default_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bool lock_local_structure)
+bool eng1_mm_default_bt::SetTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4, f64 opt, f64 fc, bool lock_local_structure)
 {
+	const i32s ind_bt3 = FindTorsion(a1, a2, a3, a4);
+	
 	if (ind_bt3 < 0) return false;
 	if (ind_bt3 >= (i32s) bt3_vector.size()) return false;
 	
@@ -627,6 +635,15 @@ bool eng1_mm_default_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, boo
 	return true;
 }
 
+bool eng1_mm_default_bt::RemoveTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4)
+{
+	// this is not yet properly implemented...
+	// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	// what to do if lock_local_structure was set??? then must unlock all...
+	
+	return false;
+}
+
 void eng1_mm_default_bt::ComputeBT1(i32u p1)
 {
 	energy_bt1 = 0.0;
@@ -675,7 +692,14 @@ void eng1_mm_default_bt::ComputeBT1(i32u p1)
 		f64 t2b = bt1_vector[n1].fc * t2a * t2a;
 		
 		energy_bt1 += t2b;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solute += t2b;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_B_bs, t2b);
+		}
 		
 		if (p1 > 0)
 		{
@@ -768,7 +792,16 @@ void eng1_mm_default_bt::ComputeBT2(i32u p1)
 		}
 		
 		energy_bt2 += t2a;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2a; else E_solute += t2a;
+		
+		if (ECOMPstore != NULL)
+		{
+			vector<int> gv;
+			gv.push_back(atmtab[atmi[0]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[1]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[2]]->ecomp_grp_i);
+			
+			ecomp_AddStoreX(gv, ECOMP_DATA_IND_B_ab, t2a);
+		}
 		
 		if (p1 > 0)
 		{
@@ -893,7 +926,17 @@ void eng1_mm_default_bt::ComputeBT3(i32u p1)
 		}
 		
 		energy_bt3 += t9a;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t9a; else E_solute += t9a;
+		
+		if (ECOMPstore != NULL)
+		{
+			vector<int> gv;
+			gv.push_back(atmtab[atmi[0]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[1]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[2]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[3]]->ecomp_grp_i);
+			
+			ecomp_AddStoreX(gv, ECOMP_DATA_IND_B_ti, t9a);
+		}
 		
 		if (p1 > 0)
 		{
@@ -982,7 +1025,17 @@ void eng1_mm_default_bt::ComputeBT4(i32u p1)
 		f64 t2a = bt4_vector[n1].fc * t1f * t1f;
 		
 		energy_bt4 += t2a;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2a; else E_solute += t2a;
+		
+		if (ECOMPstore != NULL)
+		{
+			vector<int> gv;
+			gv.push_back(atmtab[atmi[0]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[1]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[2]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[3]]->ecomp_grp_i);
+			
+			ecomp_AddStoreX(gv, ECOMP_DATA_IND_B_ti, t2a);
+		}
 		
 		if (p1 > 0)
 		{
@@ -1067,7 +1120,8 @@ eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
 	
-	ostream * ostr = default_tables::GetInstance()->ostr;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 	// also check engine::bp_center!!!
 	// also check engine::bp_center!!!
@@ -1096,7 +1150,9 @@ eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
 				else test++;
 			}
 			
-			if (test == range_cr1[ind1 + 1])	// if this is true, the atom are not in 1-2 or 1-3 positions.
+			// if this is true, then the atoms are not 1-2 or 1-3 related.
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			if (test == range_cr1[ind1 + 1])
 			{
 				test = range_cr2[ind1];
 				while (test < range_cr2[ind1 + 1])
@@ -1105,7 +1161,10 @@ eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
 					else test++;
 				}
 				
-				bool is14 = (test != range_cr2[ind1 + 1]);	// if this is true, the atoms are 1-4 related.
+				// if this is true, then the atoms are 1-4 related.
+				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+				const bool is14 = (test != range_cr2[ind1 + 1]);
+//	if (is14) cout << "DEBUG ; is 1-4 : " << ind1 << " " << ind2 << endl;
 				
 				mm_default_nbt1 newnbt1;
 				newnbt1.atmi[0] = ind1;		// this is a local index...
@@ -1116,10 +1175,12 @@ eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
 				{
 					success = default_tables::GetInstance()->e_Init(this, & newnbt1, is14);
 				}
-				else
+				
+				if (success != true)
 				{
-					// see also eng1_mm_default_nbt_mim::UpdateTerms() ; should be the same!!!
-					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					
+			// see also eng1_mm_default_nbt_mim::UpdateTerms() ; should be the same!!!
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 					
 					bool errors = false;
 					const default_at * at;
@@ -1165,11 +1226,11 @@ eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
 	// report possible errors...
 	
 	i32s total_err = nbt1_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		GetSetup()->GetModel()->PrintToLog(str.str().c_str());
 	}
 }
 
@@ -1237,7 +1298,8 @@ void eng1_mm_default_nbt_bp::ComputeNBT1(i32u p1)
 			f64 t2b = fc * t2a * t2a;
 			
 			energy_bt1 += t2b;
-if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solute += t2b;
+			
+			// do not add to ECOMP!
 			
 			if (p1 > 0)
 			{
@@ -1306,11 +1368,16 @@ if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solut
 		f64 t6b = nbt1_vector[n1].qq / t1c;
 		energy_nbt1b += t6b;
 		
-		f64 tote = t6a + t6b;
-int class1 = (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-int class2 = (atmtab[atmi[1]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-if (class1 == class2) { if (class1) E_solvent += tote; else E_solute += tote; }
-else E_solusolv += tote;
+	//	f64 tote = t6a + t6b;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_lj, t6a);
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_es, t6b);
+		}
 		
 		if (p1 > 0)
 		{
@@ -1346,7 +1413,6 @@ eng1_mm_default_nbt_mim::eng1_mm_default_nbt_mim(setup * p1, i32u p2) :
 	sw2 = shft1 = mindim - 0.2;
 	
 	limit = mindim;
-	update_neighbor_list = true;
 	
 	// calculate the actual values...
 	
@@ -1361,6 +1427,8 @@ eng1_mm_default_nbt_mim::eng1_mm_default_nbt_mim(setup * p1, i32u p2) :
 	limit = limit * limit;
 	
 	nbt1_vector.reserve(250000);
+	
+	RequestNeighborListUpdate();
 }
 
 eng1_mm_default_nbt_mim::~eng1_mm_default_nbt_mim(void)
@@ -1511,11 +1579,16 @@ void eng1_mm_default_nbt_mim::ComputeNBT1(i32u p1)
 		
 		energy_nbt1b += t6b * t4x;
 		
-		f64 tote = t6a * t3x + t6b * t4x;
-int class1 = (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-int class2 = (atmtab[atmi[1]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-if (class1 == class2) { if (class1) E_solvent += tote; else E_solute += tote; }
-else E_solusolv += tote;
+	//	f64 tote = t6a * t3x + t6b * t4x;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_lj, (t6a * t3x));
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_es, (t6b * t4x));
+		}
 		
 		if (p1 > 0)
 		{
@@ -1558,7 +1631,8 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
 	
-	ostream * ostr = default_tables::GetInstance()->ostr;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 	nbt1_vector.resize(0);
 	
@@ -1576,6 +1650,8 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 				else test++;
 			}
 			
+			// if this is true, then the atoms are not 1-2 or 1-3 related.
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 			if (test == range_cr1[ind1 + 1])
 			{
 				f64 t1a; f64 t1b = 0.0;
@@ -1646,7 +1722,10 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 					else test++;
 				}
 				
-				bool is14 = (test != range_cr2[ind1 + 1]);
+				// if this is true, then the atoms are 1-4 related.
+				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+				const bool is14 = (test != range_cr2[ind1 + 1]);
+//	if (is14) cout << "DEBUG ; is 1-4 : " << ind1 << " " << ind2 << endl;
 				
 				mm_default_nbt1 newnbt1;
 				newnbt1.atmi[0] = ind1;
@@ -1657,10 +1736,12 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 				{
 					success = default_tables::GetInstance()->e_Init(this, & newnbt1, is14);
 				}
-				else
+				
+				if (success != true)
 				{
-					// see also eng1_mm_default_nbt_bp ctor ; should be the same!!!
-					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					
+			// see also eng1_mm_default_nbt_bp ctor ; should be the same!!!
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 					
 					bool errors = false;
 					const default_at * at;
@@ -1706,11 +1787,11 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 	// report possible errors...
 	
 	i32s total_err = nbt1_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		GetSetup()->GetModel()->PrintToLog(str.str().c_str());
 	}
 }
 
