@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "eng1_mm_prmfit.h"
 
 #include "v3d.h"
@@ -29,7 +30,8 @@ eng1_mm_prmfit::eng1_mm_prmfit(setup * p1, i32u p2, prmfit_tables & tab) : engin
 	tab.UpdateTypes(GetSetup());
 	tab.UpdateCharges(GetSetup());
 	
-	ostream * ostr = tab.ostr;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 	// set default sizes to containers. does it have any practical effects???
 	
@@ -93,7 +95,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	query.atmtp[1] = atmtab[newbt1.atmi[1]]->atmtp;
 	query.bndtp = bndtab[n1]->bt.GetValue();
 	
-	tab.DoParamSearch(& query);
+	tab.DoParamSearch(& query, GetSetup()->GetModel());
 	if (query.index == NOT_DEFINED) bt1_err++;
 	
 		newbt1.opt = query.opt;
@@ -124,7 +126,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	
 	for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 	{
-		if (atmtab[n1]->cr_list.size() < 2) continue;
+		if (atmtab[n1]->GetConnRecCount() < 2) continue;
 		
 		// central atom is known, now find all possible combinations of bonds...
 		
@@ -132,9 +134,9 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 		iter_cl ita; iter_cl rnga[2];
 		iter_cl itb; iter_cl rngb[2];
 		
-		rngb[1] = atmtab[n1]->cr_list.end();
+		rngb[1] = atmtab[n1]->GetConnRecsEnd();
 		
-		rnga[0] = atmtab[n1]->cr_list.begin();
+		rnga[0] = atmtab[n1]->GetConnRecsBegin();
 		rnga[1] = rngb[1]; rnga[1]--;
 		
 		for (ita = rnga[0];ita != rnga[1];ita++)
@@ -162,7 +164,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	query.bndtp[0] = (* ita).bndr->bt.GetValue();
 	query.bndtp[1] = (* itb).bndr->bt.GetValue();
 	
-	tab.DoParamSearch(& query);
+	tab.DoParamSearch(& query, GetSetup()->GetModel());
 	if (query.index == NOT_DEFINED) bt2_err++;
 	
 				newbt2.opt = query.opt;
@@ -190,7 +192,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	
 	for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 	{
-		if (atmtab[n1]->cr_list.size() < 2) continue;
+		if (atmtab[n1]->GetConnRecCount() < 2) continue;
 		
 		// skip if default geometry is marked linear for this atomtype...
 		// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -198,12 +200,12 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	//	if (tp1 == NULL) { cout << "GetAtomType() failed!" << endl; exit(EXIT_FAILURE); }
 		if (tp1 != NULL && (tp1->flags & 3) == 1) continue;	// linear geometry -> undefined torsion.
 		
-		for (iter_cl it2 = atmtab[n1]->cr_list.begin();it2 != atmtab[n1]->cr_list.end();it2++)
+		for (iter_cl it2 = atmtab[n1]->GetConnRecsBegin();it2 != atmtab[n1]->GetConnRecsEnd();it2++)
 		{
 			bool another = (atmtab[n1] == (* it2).bndr->atmr[0]);
 			atom * atmr = (* it2).bndr->atmr[another];
 			
-			if (atmr->cr_list.size() < 2) continue;			// no torsion...
+			if (atmr->GetConnRecCount() < 2) continue;			// no torsion...
 			if (atmr->varind > atmtab[n1]->varind) continue;	// skip duplicates...
 			
 			// skip if default geometry is marked linear for this atomtype...
@@ -215,7 +217,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 			// central atoms are known, now find all possible combinations of bonds...
 			
 			vector<i32s> ind1a; vector<bool> dir1a;		// search for the 1st group...
-			for (iter_cl it3 = atmtab[n1]->cr_list.begin();it3 != atmtab[n1]->cr_list.end();it3++)
+			for (iter_cl it3 = atmtab[n1]->GetConnRecsBegin();it3 != atmtab[n1]->GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -243,7 +245,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 			}
 			
 			vector<i32s> ind1b; vector<bool> dir1b;		// search for the 2nd group...
-			for (iter_cl it3 = atmr->cr_list.begin();it3 != atmr->cr_list.end();it3++)
+			for (iter_cl it3 = atmr->GetConnRecsBegin();it3 != atmr->GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -320,7 +322,7 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 	query.bndtp[1] = (* itb2).bt.GetValue();
 	query.bndtp[2] = (* itb3).bt.GetValue();
 	
-	tab.DoParamSearch(& query);
+	tab.DoParamSearch(& query, GetSetup()->GetModel());
 	if (query.index == NOT_DEFINED) bt3_err++;
 					
 					newbt3.fc1 = query.fc1;
@@ -353,10 +355,10 @@ if (!GetSetup()->GetModel()->IsIndexClean())	// obsolete???
 		const prmfit_at * tp = tab.GetAtomType(atmtab[n1]->atmtp);
 		if (!tp || (tp != NULL && !(tp->flags & 16))) continue;
 		
-		if (atmtab[n1]->cr_list.size() != 3) continue;
+		if (atmtab[n1]->GetConnRecCount() != 3) continue;
 		
 		crec * crtab[3];
-		iter_cl iter = atmtab[n1]->cr_list.begin();
+		iter_cl iter = atmtab[n1]->GetConnRecsBegin();
 		crtab[0] = & (* iter); iter++;
 		crtab[1] = & (* iter); iter++;
 		crtab[2] = & (* iter);
@@ -439,7 +441,7 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	query.bndtp[1] = crtab[(n2 + 1) % 3]->bndr->bt.GetValue();
 	query.bndtp[2] = crtab[(n2 + 2) % 3]->bndr->bt.GetValue();
 	
-	tab.DoParamSearch(& query);
+	tab.DoParamSearch(& query, GetSetup()->GetModel());
 	if (query.index == NOT_DEFINED) bt4_err++;
 	
 			newbt4.opt = query.opt;
@@ -461,11 +463,11 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	// report possible errors...
 	
 	total_err = bt1_err + bt2_err + bt3_err + bt4_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
+		GetSetup()->GetModel()->PrintToLog(str.str().c_str());
 	}
 	
 /*##############################################*/
@@ -495,6 +497,7 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 				}
 				
 				bool is14 = (test != range_cr2[ind1 + 1]);
+//	if (is14) cout << "DEBUG ; is 1-4 : " << ind1 << " " << ind2 << endl;
 				
 				mm_prmfit_nbt1 newnbt1;
 				newnbt1.atmi[0] = ind1;		// this is a local index...
@@ -546,11 +549,11 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	// report possible errors...
 	
 	total_err = nbt1_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		GetSetup()->GetModel()->PrintToLog(str.str().c_str());
 	}
 }
 
@@ -608,8 +611,10 @@ i32s eng1_mm_prmfit::FindTorsion(atom * a1, atom * a2, atom * a3, atom * a4)
 	return NOT_DEFINED;
 }
 
-bool eng1_mm_prmfit::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bool lock_local_structure)
+bool eng1_mm_prmfit::SetTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4, f64 opt, f64 fc, bool lock_local_structure)
 {
+	const i32s ind_bt3 = FindTorsion(a1, a2, a3, a4);
+	
 	if (ind_bt3 < 0) return false;
 	if (ind_bt3 >= (i32s) bt3_vector.size()) return false;
 	
@@ -670,6 +675,15 @@ bool eng1_mm_prmfit::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bool lo
 	return true;
 }
 
+bool eng1_mm_prmfit::RemoveTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4)
+{
+	// this is not yet properly implemented...
+	// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	// what to do if lock_local_structure was set??? then must unlock all...
+	
+	return false;
+}
+
 void eng1_mm_prmfit::ComputeBT1(i32u p1)
 {
 	energy_bt1 = 0.0;
