@@ -10,8 +10,6 @@
 #ifndef ENG1_MM_PRMFIT_H
 #define ENG1_MM_PRMFIT_H
 
-//#include "libghemicalconfig2.h"
-
 struct mm_prmfit_bt1;		// bond stretching
 struct mm_prmfit_bt2;		// angle bending
 struct mm_prmfit_bt3;		// torsion terms
@@ -155,8 +153,9 @@ class eng1_mm_prmfit : public eng1_mm
 // strategy: the tab_mm_prmfit/eng1_mm_prmfit classes are different from the default ones,
 // ^^^^^^^^^ and it seems to be OK; compatibility at parameter-file-level seems to be enough!
 	
-	i32s FindTorsion(atom *, atom *, atom *, atom *);	// virtual
-	bool SetTorsionConstraint(i32s, f64, f64, bool);	// virtual
+	i32s FindTorsion(atom *, atom *, atom *, atom *);
+	bool SetTorsionConstraint(atom *, atom *, atom *, atom *, f64, f64, bool);	// virtual
+	bool RemoveTorsionConstraint(atom *, atom *, atom *, atom *);			// virtual
 	
 	protected:
 	
