@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "eng1_mm_tripos52.h"
 
 #include "v3d.h"
@@ -36,8 +37,8 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 	model * mdl = GetSetup()->GetModel();
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 	
-//	ostream * ostr = tripos52_tables::GetInstance()->ostr;
-	ostream * ostr = & cout;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 /*##############################################*/
 /*##############################################*/
@@ -97,7 +98,7 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 	
 	for (iter_al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
 	{
-		if ((* it1).cr_list.size() < 2) continue;
+		if ((* it1).GetConnRecCount() < 2) continue;
 		
 		// central atom is known, now find all possible combinations of bonds...
 		
@@ -105,9 +106,9 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 		iter_cl ita; iter_cl rnga[2];
 		iter_cl itb; iter_cl rngb[2];
 		
-		rngb[1] = (* it1).cr_list.end();
+		rngb[1] = (* it1).GetConnRecsEnd();
 		
-		rnga[0] = (* it1).cr_list.begin();
+		rnga[0] = (* it1).GetConnRecsBegin();
 		rnga[1] = rngb[1]; rnga[1]--;
 		
 		for (ita = rnga[0];ita != rnga[1];ita++)
@@ -156,20 +157,20 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 	
 	for (iter_al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
 	{
-		if ((* it1).cr_list.size() < 2) continue;
+		if ((* it1).GetConnRecCount() < 2) continue;
 		
-		for (iter_cl it2 = (* it1).cr_list.begin();it2 != (* it1).cr_list.end();it2++)
+		for (iter_cl it2 = (* it1).GetConnRecsBegin();it2 != (* it1).GetConnRecsEnd();it2++)
 		{
 			bool another = (& (* it1) == (* it2).bndr->atmr[0]);
 			atom * atmr = (* it2).bndr->atmr[another];
 			
-			if (atmr->cr_list.size() < 2) continue;
+			if (atmr->GetConnRecCount() < 2) continue;
 			if (atmr->index > (* it1).index) continue;
 			
 			// central atoms are known, now find all possible combinations of bonds...
 			
 			vector<i32s> ind1a; vector<bool> dir1a;		// search for the 1st group...
-			for (iter_cl it3 = (* it1).cr_list.begin();it3 != (* it1).cr_list.end();it3++)
+			for (iter_cl it3 = (* it1).GetConnRecsBegin();it3 != (* it1).GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -199,7 +200,7 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 			}
 			
 			vector<i32s> ind1b; vector<bool> dir1b;		// search for the 2nd group...
-			for (iter_cl it3 = atmr->cr_list.begin();it3 != atmr->cr_list.end();it3++)
+			for (iter_cl it3 = atmr->GetConnRecsBegin();it3 != atmr->GetConnRecsEnd();it3++)
 			{
 				if ((* it3) == (* it2)) continue;
 				
@@ -259,16 +260,6 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 					// easiest way to get bondtypes is to find the bonds...
 					// easiest way to get bondtypes is to find the bonds...
 					
-/*	bond tmpb1 = bond(index[newbt3.atmi[0]], index[newbt3.atmi[1]], bondtype());
-	iter_bl itb1 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), tmpb1);
-	
-	bond tmpb2 = bond(index[newbt3.atmi[1]], index[newbt3.atmi[2]], bondtype());
-	iter_bl itb2 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), tmpb2);
-	
-	bond tmpb3 = bond(index[newbt3.atmi[2]], index[newbt3.atmi[3]], bondtype());
-	iter_bl itb3 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), tmpb3);	*/
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
 	bond tmpb1 = bond(atmtab[newbt3.atmi[0]], atmtab[newbt3.atmi[1]], bondtype());
 	iter_bl itb1 = find(GetSetup()->GetModel()->GetBondsBegin(), GetSetup()->GetModel()->GetBondsEnd(), tmpb1);
 	
@@ -277,7 +268,7 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 	
 	bond tmpb3 = bond(atmtab[newbt3.atmi[2]], atmtab[newbt3.atmi[3]], bondtype());
 	iter_bl itb3 = find(GetSetup()->GetModel()->GetBondsBegin(), GetSetup()->GetModel()->GetBondsEnd(), tmpb3);
-	
+					
 					i32s bt[3];
 					bt[0] = (* itb1).bt.GetValue();
 					bt[1] = (* itb2).bt.GetValue();
@@ -285,7 +276,7 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 					
 	bt3_err += !tripos52_tables::GetInstance()->Init(this, & newbt3, bt);
 //	if (newbt3.k == 0.0) continue;		// these have bad optimal geometries -> SKIP!!! constraints???
-
+					
 					newbt3.constraint = false;
 					bt3_vector.push_back(newbt3);
 				}
@@ -305,11 +296,11 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
 	// report possible errors...
 	
 	i32s total_err = bt1_err + bt2_err + bt3_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		mdl->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
+		mdl->PrintToLog(str.str().c_str());
 	}
 }
 
@@ -321,30 +312,6 @@ eng1_mm_tripos52_bt::~eng1_mm_tripos52_bt(void)
 
 i32s eng1_mm_tripos52_bt::FindTorsion(atom * a1, atom * a2, atom * a3, atom * a4)
 {
-/*	i32s tmp[4] = { a1->index, a2->index, a3->index, a4->index };
-	
-	for (i32s n1 = 0;n1 < (i32s) bt3_vector.size();n1++)
-	{
-		bool test;	// since torsion is the same in both directions, we can ignore direction...
-		
-		test = true;
-		if (bt3_vector[n1].atmi[0] != tmp[0]) test = false;
-		if (bt3_vector[n1].atmi[1] != tmp[1]) test = false;
-		if (bt3_vector[n1].atmi[2] != tmp[2]) test = false;
-		if (bt3_vector[n1].atmi[3] != tmp[3]) test = false;
-		if (test) return n1;
-		
-		test = true;
-		if (bt3_vector[n1].atmi[0] != tmp[3]) test = false;
-		if (bt3_vector[n1].atmi[1] != tmp[2]) test = false;
-		if (bt3_vector[n1].atmi[2] != tmp[1]) test = false;
-		if (bt3_vector[n1].atmi[3] != tmp[0]) test = false;
-		if (test) return n1;
-	}
-	
-	return NOT_DEFINED;	*/
-////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////
 	i32s iglob[4] = { a1->varind, a2->varind, a3->varind, a4->varind };
 	i32s iloc[4];
 	
@@ -391,8 +358,10 @@ i32s eng1_mm_tripos52_bt::FindTorsion(atom * a1, atom * a2, atom * a3, atom * a4
 	return NOT_DEFINED;
 }
 
-bool eng1_mm_tripos52_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bool lock_local_structure)
+bool eng1_mm_tripos52_bt::SetTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4, f64 opt, f64 fc, bool lock_local_structure)
 {
+	const i32s ind_bt3 = FindTorsion(a1, a2, a3, a4);
+	
 	if (ind_bt3 < 0) return false;
 	if (ind_bt3 >= (i32s) bt3_vector.size()) return false;
 	
@@ -405,13 +374,6 @@ bool eng1_mm_tripos52_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bo
 	
 	if (lock_local_structure)
 	{
-	/*	v3d<f64> v1a(crd[bt3_vector[ind_bt3].atmi[1]], crd[bt3_vector[ind_bt3].atmi[0]]);
-		v3d<f64> v1b(crd[bt3_vector[ind_bt3].atmi[1]], crd[bt3_vector[ind_bt3].atmi[2]]);
-		v3d<f64> v1c(crd[bt3_vector[ind_bt3].atmi[2]], crd[bt3_vector[ind_bt3].atmi[3]]);
-		f64 delta = opt - v1a.tor(v1b, v1c);	*/
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
-////////////////////////////////////////////////////////////////////////////////////////////////////
 		v3d<f64> v1a(& crd[l2g_mm[bt3_vector[ind_bt3].atmi[1]] * 3], & crd[l2g_mm[bt3_vector[ind_bt3].atmi[0]] * 3]);
 		v3d<f64> v1b(& crd[l2g_mm[bt3_vector[ind_bt3].atmi[1]] * 3], & crd[l2g_mm[bt3_vector[ind_bt3].atmi[2]] * 3]);
 		v3d<f64> v1c(& crd[l2g_mm[bt3_vector[ind_bt3].atmi[2]] * 3], & crd[l2g_mm[bt3_vector[ind_bt3].atmi[3]] * 3]);
@@ -435,12 +397,6 @@ bool eng1_mm_tripos52_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bo
 			
 			if (test1 || test2)
 			{
-			/*	v3d<f64> v2a(crd[bt3_vector[n1].atmi[1]], crd[bt3_vector[n1].atmi[0]]);
-				v3d<f64> v2b(crd[bt3_vector[n1].atmi[1]], crd[bt3_vector[n1].atmi[2]]);
-				v3d<f64> v2c(crd[bt3_vector[n1].atmi[2]], crd[bt3_vector[n1].atmi[3]]);
-				f64 local = v2a.tor(v2b, v2c) + delta;	*/
-	////////////////////////////////////////////////////////////////////////////////////////////////////
-	////////////////////////////////////////////////////////////////////////////////////////////////////
 				v3d<f64> v2a(& crd[l2g_mm[bt3_vector[n1].atmi[1]] * 3], & crd[l2g_mm[bt3_vector[n1].atmi[0]] * 3]);
 				v3d<f64> v2b(& crd[l2g_mm[bt3_vector[n1].atmi[1]] * 3], & crd[l2g_mm[bt3_vector[n1].atmi[2]] * 3]);
 				v3d<f64> v2c(& crd[l2g_mm[bt3_vector[n1].atmi[2]] * 3], & crd[l2g_mm[bt3_vector[n1].atmi[3]] * 3]);
@@ -463,6 +419,15 @@ bool eng1_mm_tripos52_bt::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bo
 	return true;
 }
 
+bool eng1_mm_tripos52_bt::RemoveTorsionConstraint(atom * a1, atom * a2, atom * a3, atom * a4)
+{
+	// this is not yet properly implemented...
+	// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	// what to do if lock_local_structure was set??? then must unlock all...
+	
+	return false;
+}
+
 void eng1_mm_tripos52_bt::ComputeBT1(i32u p1)
 {
 	energy_bt1 = 0.0;
@@ -487,10 +452,6 @@ void eng1_mm_tripos52_bt::ComputeBT1(i32u p1)
 		f64 t1a[3]; f64 t1b = 0.0;
 		for (i32s n2 = 0;n2 < 3;n2++)
 		{
-	//		f64 t9a = crd[atmi[0]][n2];
-	//		f64 t9b = crd[atmi[1]][n2];
-///////////////////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////////////////
 			f64 t9a = crd[l2g_mm[atmi[0]] * 3 + n2];
 			f64 t9b = crd[l2g_mm[atmi[1]] * 3 + n2];
 			
@@ -515,7 +476,14 @@ void eng1_mm_tripos52_bt::ComputeBT1(i32u p1)
 		f64 t2b = bt1_vector[n1].fc * t2a * t2a;
 		
 		energy_bt1 += t2b;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solute += t2b;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_B_bs, t2b);
+		}
 		
 		if (p1 > 0)
 		{
@@ -525,10 +493,6 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_
 			{
 				f64 t2d = bt1data[n1].dlen[0][n2] * t2c;
 				
-		//		d1[atmi[0]][n2] += t2d;
-		//		d1[atmi[1]][n2] -= t2d;
-		//////////////////////////////////////////////////////////////////////
-		//////////////////////////////////////////////////////////////////////
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t2d;
 				d1[l2g_mm[atmi[1]] * 3 + n2] -= t2d;
 			}
@@ -579,6 +543,8 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_
 		
 		energy_bt1 += t2b;
 		
+		// do not add to ECOMP!
+		
 		if (p1 > 0)
 		{
 			for (i32s n2 = 0;n2 < 3;n2++)
@@ -657,17 +623,21 @@ void eng1_mm_tripos52_bt::ComputeBT2(i32u p1)
 		}
 		
 		energy_bt2 += t2a;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2a; else E_solute += t2a;
+		
+		if (ECOMPstore != NULL)
+		{
+			vector<int> gv;
+			gv.push_back(atmtab[atmi[0]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[1]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[2]]->ecomp_grp_i);
+			
+			ecomp_AddStoreX(gv, ECOMP_DATA_IND_B_ab, t2a);
+		}
 		
 		if (p1 > 0)
 		{
 			for (i32s n2 = 0;n2 < 3;n2++)
 			{
-			//	d1[atmi[0]][n2] += bt2data[n1].dcsa[0][n2] * t2b;
-			//	d1[atmi[1]][n2] += bt2data[n1].dcsa[1][n2] * t2b;
-			//	d1[atmi[2]][n2] += bt2data[n1].dcsa[2][n2] * t2b;
-			//////////////////////////////////////////////////////////////////////7
-			//////////////////////////////////////////////////////////////////////7
 				d1[l2g_mm[atmi[0]] * 3 + n2] += bt2data[n1].dcsa[0][n2] * t2b;
 				d1[l2g_mm[atmi[1]] * 3 + n2] += bt2data[n1].dcsa[1][n2] * t2b;
 				d1[l2g_mm[atmi[2]] * 3 + n2] += bt2data[n1].dcsa[2][n2] * t2b;
@@ -792,7 +762,17 @@ void eng1_mm_tripos52_bt::ComputeBT3(i32u p1)
 		}
 		
 		energy_bt3 += t9a;
-if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t9a; else E_solute += t9a;
+		
+		if (ECOMPstore != NULL)
+		{
+			vector<int> gv;
+			gv.push_back(atmtab[atmi[0]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[1]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[2]]->ecomp_grp_i);
+			gv.push_back(atmtab[atmi[3]]->ecomp_grp_i);
+			
+			ecomp_AddStoreX(gv, ECOMP_DATA_IND_B_ti, t9a);
+		}
 		
 		if (p1 > 0)
 		{
@@ -815,20 +795,12 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t9a; else E_
 				f64 t5b = bt1data[index1[2]].dlen[dir1[2]][cp[n2][1]] * bt1data[index1[3]].dlen[dir1[3]][cp[n2][2]];
 				t5c[n2] = (t5a - t5b) / t3a;
 				
-			//	d1[atmi[0]][n2] += t4c[n2] * t9b;
-			//	d1[atmi[3]][n2] += t5c[n2] * t9b;
-			//////////////////////////////////////////////////////////
-			//////////////////////////////////////////////////////////
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t4c[n2] * t9b;
 				d1[l2g_mm[atmi[3]] * 3 + n2] += t5c[n2] * t9b;
 				
 				t6a[n2] = (t2b - 1.0) * t4c[n2] - t3b * t5c[n2];
 				t7a[n2] = (t3b - 1.0) * t5c[n2] - t2b * t4c[n2];
 				
-			//	d1[atmi[1]][n2] += t6a[n2] * t9b;
-			//	d1[atmi[2]][n2] += t7a[n2] * t9b;
-			//////////////////////////////////////////////////////////
-			//////////////////////////////////////////////////////////
 				d1[l2g_mm[atmi[1]] * 3 + n2] += t6a[n2] * t9b;
 				d1[l2g_mm[atmi[2]] * 3 + n2] += t7a[n2] * t9b;
 			}
@@ -852,8 +824,8 @@ eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
 	
-//	ostream * ostr = tripos52_tables::GetInstance()->ostr;
-	ostream * ostr = & cout;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 	// also check engine::bp_center!!!
 	// also check engine::bp_center!!!
@@ -871,39 +843,6 @@ eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
 	if (ostr != NULL) (* ostr) << "creating nbt1-terms: ";
 	i32s nbt1_err = 0;
 	
-/*	for (i32s ind1 = 0;ind1 < natm - 1;ind1++)
-	{
-		for (i32s ind2 = ind1 + 1;ind2 < natm;ind2++)
-		{
-			i32s test = range_cr1[ind1];
-			while (test < range_cr1[ind1 + 1])
-			{
-				if (cr1[test] == ind2) break;
-				else test++;
-			}
-			
-			if (test == range_cr1[ind1 + 1])
-			{
-				test = range_cr2[ind1];
-				while (test < range_cr2[ind1 + 1])
-				{
-					if (cr2[test] == ind2) break;
-					else test++;
-				}
-				
-				bool is14 = (test != range_cr2[ind1 + 1]);
-				
-				mm1_exp1_nbt1 newnbt1;
-				newnbt1.atmi[0] = ind1;
-				newnbt1.atmi[1] = ind2;
-				
-				nbt1_err += !exp1_tables::GetInstance()->Init(this, & newnbt1, is14);
-				nbt1_vector.push_back(newnbt1);
-			}
-		}
-	}	*/
-/////////////////////////////////////////////////////
-/////////////////////////////////////////////////////
 	for (i32s ind1 = 0;ind1 < GetSetup()->GetMMAtomCount() - 1;ind1++)
 	{
 		for (i32s ind2 = ind1 + 1;ind2 < GetSetup()->GetMMAtomCount();ind2++)
@@ -915,7 +854,9 @@ eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
 				else test++;
 			}
 			
-			if (test == range_cr1[ind1 + 1])	// if this is true, the atom are not in 1-2 or 1-3 positions.
+			// if this is true, then the atoms are not 1-2 or 1-3 related.
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			if (test == range_cr1[ind1 + 1])
 			{
 				test = range_cr2[ind1];
 				while (test < range_cr2[ind1 + 1])
@@ -924,10 +865,13 @@ eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
 					else test++;
 				}
 				
-				bool is14 = (test != range_cr2[ind1 + 1]);	// if this is true, the atoms are 1-4 related.
+				// if this is true, then the atoms are 1-4 related.
+				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+				const bool is14 = (test != range_cr2[ind1 + 1]);
+//	if (is14) cout << "DEBUG ; is 1-4 : " << ind1 << " " << ind2 << endl;
 				
-				// see also eng1_mm_tripos52_nbt_mim::UpdateTerms() ; should be the same!!!
-				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			// see also eng1_mm_tripos52_nbt_mim::UpdateTerms() ; should be the same!!!
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 				
 				mm_tripos52_nbt1 newnbt1;
 				newnbt1.atmi[0] = ind1;		// this is a local index...
@@ -963,11 +907,11 @@ eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
 	// report possible errors...
 	
 	i32s total_err = nbt1_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		mdl->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		mdl->PrintToLog(str.str().c_str());
 	}
 }
 
@@ -1035,7 +979,8 @@ void eng1_mm_tripos52_nbt_bp::ComputeNBT1(i32u p1)
 			f64 t2b = fc * t2a * t2a;
 			
 			energy_bt1 += t2b;
-if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solute += t2b;
+			
+			// do not add to ECOMP!
 			
 			if (p1 > 0)
 			{
@@ -1063,10 +1008,6 @@ if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solut
 		f64 t1a[3]; f64 t1b = 0.0;
 		for (i32s n2 = 0;n2 < 3;n2++)
 		{
-//			f64 t2a = crd[atmi[0]][n2];
-//			f64 t2b = crd[atmi[1]][n2];
-/////////////////////////////////////////////////////////////////////////
-/////////////////////////////////////////////////////////////////////////
 			f64 t2a = crd[l2g_mm[atmi[0]] * 3 + n2];
 			f64 t2b = crd[l2g_mm[atmi[1]] * 3 + n2];
 			
@@ -1108,11 +1049,16 @@ if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_solut
 		
 		energy_nbt1a += t6a + t6b;
 		
-		f64 tote = t6a + t6b;
-int class1 = (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-int class2 = (atmtab[atmi[1]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-if (class1 == class2) { if (class1) E_solvent += tote; else E_solute += tote; }
-else E_solusolv += tote;
+	//	f64 tote = t6a + t6b;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_lj, t6a);
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_es, t6b);
+		}
 		
 		if (p1 > 0)
 		{
@@ -1126,10 +1072,6 @@ else E_solusolv += tote;
 			{
 				f64 t9b = (t1a[n2] / t1c) * t9a;
 				
-		//		d1[atmi[0]][n2] += t9b;
-		//		d1[atmi[1]][n2] -= t9b;
-		///////////////////////////////////////////////////////
-		///////////////////////////////////////////////////////
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t9b;
 				d1[l2g_mm[atmi[1]] * 3 + n2] -= t9b;
 			}
@@ -1152,7 +1094,6 @@ eng1_mm_tripos52_nbt_mim::eng1_mm_tripos52_nbt_mim(setup * p1, i32u p2) :
 	sw2 = shft1 = mindim - 0.2;
 	
 	limit = mindim;
-	update_neighbor_list = true;
 	
 	// calculate the actual values...
 	
@@ -1168,6 +1109,8 @@ eng1_mm_tripos52_nbt_mim::eng1_mm_tripos52_nbt_mim(setup * p1, i32u p2) :
 	
 	nbt1_vector.reserve(250000);
 	
+	RequestNeighborListUpdate();
+	
 	// check from the distance constraints if there are any skip requests.
 	
 	int nskip = 0;
@@ -1210,10 +1153,6 @@ void eng1_mm_tripos52_nbt_mim::ComputeNBT1(i32u p1)
 		f64 t1a[3]; f64 t1b = 0.0;
 		for (i32s n2 = 0;n2 < 3;n2++)
 		{
-	//		f64 t2a = crd[atmi[0]][n2];
-	//		f64 t2b = crd[atmi[1]][n2];
-	///////////////////////////////////////////////////////
-	///////////////////////////////////////////////////////
 			f64 t2a = crd[l2g_mm[atmi[0]] * 3 + n2];
 			f64 t2b = crd[l2g_mm[atmi[1]] * 3 + n2];
 			
@@ -1297,11 +1236,16 @@ void eng1_mm_tripos52_nbt_mim::ComputeNBT1(i32u p1)
 		
 		energy_nbt1b += t6b * t4x;
 		
-		f64 tote = t6a * t3x + t6b * t4x;
-int class1 = (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-int class2 = (atmtab[atmi[1]]->flags & ATOMFLAG_IS_SOLVENT_ATOM);
-if (class1 == class2) { if (class1) E_solvent += tote; else E_solute += tote; }
-else E_solusolv += tote;
+	//	f64 tote = t6a * t3x + t6b * t4x;
+		
+		if (ECOMPstore != NULL)
+		{
+			const int iA = atmtab[atmi[0]]->ecomp_grp_i;
+			const int iB = atmtab[atmi[1]]->ecomp_grp_i;
+			
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_lj, (t6a * t3x));
+			ecomp_AddStore2(iA, iB, ECOMP_DATA_IND_NB_es, (t6b * t4x));
+		}
 		
 		if (p1 > 0)
 		{
@@ -1319,10 +1263,6 @@ else E_solusolv += tote;
 			{
 				f64 t9d = (t1a[n2] / t1c) * t9c;
 				
-	//			d1[atmi[0]][n2] += t9d;
-	//			d1[atmi[1]][n2] -= t9d;
-	/////////////////////////////////////////////////////////////////
-	/////////////////////////////////////////////////////////////////
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t9d;
 				d1[l2g_mm[atmi[1]] * 3 + n2] -= t9d;
 			}
@@ -1338,8 +1278,8 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
 	
-//	ostream * ostr = tripos52_tables::GetInstance()->ostr;
-	ostream * ostr = & cout;
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
 	
 	nbt1_vector.resize(0);
 	
@@ -1357,7 +1297,9 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 				else test++;
 			}
 			
-			if (test == range_cr1[ind1 + 1])	// if this is true, the atom are not in 1-2 or 1-3 positions.
+			// if this is true, then the atoms are not 1-2 or 1-3 related.
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			if (test == range_cr1[ind1 + 1])
 			{
 				f64 t1a; f64 t1b = 0.0;
 				for (i32s n1 = 0;n1 < 3;n1++)
@@ -1388,10 +1330,13 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 					else test++;
 				}
 				
-				bool is14 = (test != range_cr2[ind1 + 1]);	// if this is true, the atoms are 1-4 related.
+				// if this is true, then the atoms are 1-4 related.
+				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+				const bool is14 = (test != range_cr2[ind1 + 1]);
+//	if (is14) cout << "DEBUG ; is 1-4 : " << ind1 << " " << ind2 << endl;
 				
-				// see also eng1_mm_tripos52_nbt_bp ctor ; should be the same!!!
-				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+			// see also eng1_mm_tripos52_nbt_bp ctor ; should be the same!!!
+			// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 				
 				mm_tripos52_nbt1 newnbt1;
 				newnbt1.atmi[0] = ind1;		// this is a local index...
@@ -1406,11 +1351,11 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 	// report possible errors...
 	
 	i32s total_err = nbt1_err;
-	if (total_err)
+	if (total_err && GetSetup()->GetModel()->verbosity >= 2)
 	{
-		ostringstream str1;
-		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		mdl->PrintToLog(str1.str().c_str());
+		ostringstream str;
+		str << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
+		mdl->PrintToLog(str.str().c_str());
 	}
 }
 
