@@ -10,8 +10,6 @@
 #ifndef ENG1_QM_H
 #define ENG1_QM_H
 
-//#include "libghemialconfig2.h"
-
 class setup1_qm;
 
 class eng1_qm;
@@ -114,8 +112,8 @@ class eng1_qm : virtual public engine
 	eng1_qm(setup *, i32u);
 	virtual ~eng1_qm(void);
 	
-	virtual i32s FindTorsion(atom *, atom *, atom *, atom *) { cout << "BUG: FindTorsion() is not yet implemented at eng1_qm!" << endl; exit(EXIT_FAILURE); }
-	virtual bool SetTorsionConstraint(i32s, f64, f64, bool) { cout << "BUG: SetTorsionConstraint() is not yet implemented at eng1_qm!" << endl; exit(EXIT_FAILURE); }
+	virtual bool SetTorsionConstraint(atom *, atom *, atom *, atom *, f64, f64, bool) { cout << "callEXIT : SetTorsionConstraint() is not yet implemented at eng1_qm!" << endl; exit(EXIT_FAILURE); }
+	virtual bool RemoveTorsionConstraint(atom *, atom *, atom *, atom *) { cout << "callEXIT : RemoveTorsionConstraint() is not yet implemented at eng1_qm!" << endl; exit(EXIT_FAILURE); }
 	
 	virtual fGL GetVDWSurf(fGL *, fGL *);	// virtual
 };
