@@ -10,8 +10,6 @@
 #ifndef ENG1_QM_MOPAC_H
 #define ENG1_QM_MOPAC_H
 
-#include "libghemicalconfig2.h"
-
 #ifdef ENABLE_MOPAC7
 
 class eng1_qm_mopac;
@@ -35,6 +33,8 @@ class eng1_qm_mopac : public eng1_qm
 	eng1_qm_mopac(setup *, i32u, i32u);
 	~eng1_qm_mopac(void);
 	
+	void FixMeLaterTSS(void);
+	
 	static eng1_qm_mopac * GetLock(void) { return mopac_lock; }
 	
 	i32s GetOrbitalCount(void);		// virtual
