@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "eng1_qm_mpqc.h"	// config.h is here -> we get ENABLE-macros here...
 
 #ifdef ENABLE_MPQC
@@ -14,20 +15,20 @@
 #include <sstream>
 using namespace std;
 
-#include "util/misc/regtime.h"			// RegionTimer
-#include "util/misc/formio.h"			// SCFormIO
+#include <util/misc/regtime.h>			// RegionTimer
+#include <util/misc/formio.h>			// SCFormIO
 
-#include "math/scmat/vector3.h"			// SCVector3
+#include <math/scmat/vector3.h>			// SCVector3
 
-#include "chemistry/molecule/formula.h"		// MolecularFormula
+#include <chemistry/molecule/formula.h>		// MolecularFormula
 
-#include "chemistry/qc/basis/obint.h"		// PointChargeData
+#include <chemistry/qc/basis/obint.h>		// PointChargeData
 
 // force linkages!!! this will make sure that all important stuff will be linked from the libraries...
 // force linkages!!! this will make sure that all important stuff will be linked from the libraries...
 // force linkages!!! this will make sure that all important stuff will be linked from the libraries...
 
-#include "chemistry/qc/scf/linkage.h"
+#include <chemistry/qc/scf/linkage.h>
 
 /*################################################################################################*/
 
@@ -256,10 +257,6 @@ i32s eng1_qm_mpqc::GetElectronCount(void)
 
 void eng1_qm_mpqc::Compute(i32u p1, bool)
 {
-	E_solute = 0.0;		// re-think this for eng2_xx_yy classes!!!
-	E_solvent = 0.0;	// re-think this for eng2_xx_yy classes!!!
-	E_solusolv = 0.0;	// re-think this for eng2_xx_yy classes!!!
-	
 	const double cf1 = 18.897162;	// conversion factor for [nm] -> [bohr]
 	const double cf2 = 2625.5;	// conversion factor for [Hartree] -> [kJ/mol]
 	
