@@ -10,7 +10,7 @@
 #ifndef ENG2_QM_MM_H
 #define ENG2_QM_MM_H
 
-#include "libghemicalconfig2.h"
+#include "libghemical-features.h"
 
 class setup2_qm_mm;
 
@@ -76,11 +76,11 @@ class eng2_qm_mm_mopac : public eng1_qm_mopac, public eng1_mm_default_bp
 	
 	void Compute(i32u, bool = false);	// virtual
 	
-	i32s FindTorsion(atom *, atom *, atom *, atom *) { cout << "BUG: FindTorsion() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
-	bool SetTorsionConstraint(i32s, f64, f64, bool) { cout << "BUG: SetTorsionConstraint() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
+	virtual bool SetTorsionConstraint(atom *, atom *, atom *, atom *, f64, f64, bool) { cout << "callEXIT : SetTorsionConstraint() is not yet implemented at eng2_qm_mm_mopac!" << endl; exit(EXIT_FAILURE); }
+	virtual bool RemoveTorsionConstraint(atom *, atom *, atom *, atom *) { cout << "callEXIT : RemoveTorsionConstraint() is not yet implemented at eng2_qm_mm_mopac!" << endl; exit(EXIT_FAILURE); }
 	
 	void SetupPlotting(void);	// virtual
-
+	
 	fGL GetVDWSurf(fGL *, fGL *);	// virtual
 	
 	fGL GetESP(fGL *, fGL *);	// virtual
@@ -113,8 +113,8 @@ class eng2_qm_mm_mpqc : public eng1_qm_mpqc, public eng1_mm_default_bp
 	
 	void Compute(i32u, bool = false);	// virtual
 	
-	i32s FindTorsion(atom *, atom *, atom *, atom *) { cout << "BUG: FindTorsion() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
-	bool SetTorsionConstraint(i32s, f64, f64, bool) { cout << "BUG: SetTorsionConstraint() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
+	virtual bool SetTorsionConstraint(atom *, atom *, atom *, atom *, f64, f64, bool) { cout << "callEXIT : SetTorsionConstraint() is not yet implemented at eng2_qm_mm_mpqc!" << endl; exit(EXIT_FAILURE); }
+	virtual bool RemoveTorsionConstraint(atom *, atom *, atom *, atom *) { cout << "callEXIT : RemoveTorsionConstraint() is not yet implemented at eng2_qm_mm_mpqc!" << endl; exit(EXIT_FAILURE); }
 	
 	void SetupPlotting(void);	// virtual
 
