@@ -7,12 +7,16 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "engine.h"
 
 #include "v3d.h"
 
 #include "model.h"
 
+#include <vector>
+using namespace std;
+
 /*################################################################################################*/
 
 setup::setup(model * p1)
@@ -343,14 +347,46 @@ engine::engine(setup * p1, i32u p2)
 	virial[0] = 0.0;
 	virial[1] = 0.0;
 	virial[2] = 0.0;
+	
+	update_neighbor_list = false;
+	
+	ECOMPcycles = 0;
+	ECOMPstore_size = 0;
+	ECOMPstore = NULL;
+	
+	if (stp->GetModel()->ecomp_enabled)
+	{
+		const int n = stp->GetModel()->ecomp_grp_names.size();
+		ECOMPstore_size = n * (n + 1) / 2;
+		
+		ECOMPstore = new ecomp_data[ECOMPstore_size];
+		
+		ecomp_Reset();
+	}
 }
 
 engine::~engine(void)
 {
 	delete[] crd;
+	crd = NULL;
 	
-	if (d1 != NULL) delete[] d1;
-	if (d2 != NULL) delete[] d2;
+	if (d1 != NULL)
+	{
+		delete[] d1;
+		d1 = NULL;
+	}
+	
+	if (d2 != NULL)
+	{
+		delete[] d2;
+		d2 = NULL;
+	}
+	
+	if (ECOMPstore != NULL)
+	{
+		delete[] ECOMPstore;
+		ECOMPstore = NULL;
+	}
 }
 
 void engine::Check(i32u)
@@ -412,6 +448,124 @@ void engine::DoSHAKE(void)
 {
 }
 
+f64 engine::GetEnergy(void)
+{
+	return energy;
+}
+
+f64 engine::ReadCRD(int i)
+{
+	if (i < 0) return 0.0;
+	if (i >= natm * 3) return 0.0;
+	
+	return crd[i];
+}
+
+void engine::ecomp_AddCycle(void)
+{
+	ECOMPcycles++;
+}
+
+void engine::ecomp_AddStore2(int gA, int gB, int sc, double value)
+{
+	const int gLO = (gA < gB ? gA : gB);
+	const int gHI = (gA > gB ? gA : gB);
+	
+	const int index = (gHI * (gHI + 1) / 2) + gLO;
+	
+	if (index >= ECOMPstore_size)
+	{
+		cout << "callEXIT : index overflow at engine::ecomp_AddStore()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	ECOMPstore[index][sc] += value;
+}
+
+void engine::ecomp_AddStoreX(vector<int> & gv, int sc, double value)
+{
+	if (gv.size() < 1) return;
+	
+	if (gv.size() == 1)
+	{
+		ecomp_AddStore2(gv[0], gv[0], sc, value);
+		return;
+	}
+	
+	if (gv.size() == 2)
+	{
+		ecomp_AddStore2(gv[0], gv[1], sc, value);
+		return;
+	}
+	
+	vector<int> uniq;
+	uniq.push_back(gv[0]);
+	
+	for (int n1 = 1;n1 < (int) gv.size();n1++)
+	{
+		bool is_uniq = true;
+		for (int n2 = 0;n2 < (int) uniq.size();n2++)
+		{
+			if (uniq[n2] == gv[n1])
+			{
+				is_uniq = false;
+				break;
+			}
+		}
+		
+		if (is_uniq) uniq.push_back(gv[n1]);
+	}
+	
+	if (uniq.size() == 1)
+	{
+		ecomp_AddStore2(uniq[0], uniq[0], sc, value);
+	}
+	else
+	{
+		const int s = uniq.size();
+		const int n = s * (s - 1) / 2;
+		const double valX = value / (double) n;
+		
+		for (int x = 0;x < s - 1;x++)
+		{
+			for (int y = x + 1;y < s;y++)
+			{
+				ecomp_AddStore2(uniq[x], uniq[y], sc, valX);
+			}
+		}
+	}
+}
+
+void engine::ecomp_Reset(void)
+{
+	ECOMPcycles = 0;
+	
+	for (int i = 0;i < ECOMPstore_size;i++)
+	{
+		ECOMPstore[i][ECOMP_DATA_IND_B_bs] = 0.0;
+		ECOMPstore[i][ECOMP_DATA_IND_B_ab] = 0.0;
+		ECOMPstore[i][ECOMP_DATA_IND_B_ti] = 0.0;
+		ECOMPstore[i][ECOMP_DATA_IND_NB_lj] = 0.0;
+		ECOMPstore[i][ECOMP_DATA_IND_NB_es] = 0.0;
+	}
+}
+
+double engine::ecomp_ReadStore(int gA, int gB, int sc)
+{
+	const int gLO = (gA < gB ? gA : gB);
+	const int gHI = (gA > gB ? gA : gB);
+	
+	const int index = (gHI * (gHI + 1) / 2) + gLO;
+	
+	if (index >= ECOMPstore_size)
+	{
+		cout << "callEXIT : index overflow at engine::ecomp_AddStore()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	return (ECOMPstore[index][sc] / (double) ECOMPcycles);
+}
+
 /*################################################################################################*/
 
 fGL value_VDWSurf(engine * eng, fGL * crd, fGL * grad)
