@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "intcrd.h"
 
 #include "v3d.h"
@@ -66,14 +67,14 @@ intcrd::intcrd(model & p1, i32s p2, i32s p3)
 	// find the best atom (the one with least bonds) to work as an "anchor".
 	
 	iter_al anchor = range[0];
-	i32u nbonds = (* anchor).cr_list.size();
+	i32u nbonds = (* anchor).GetConnRecCount();
 	
 	for (iter_al it1 = range[0];it1 != range[1];it1++)
 	{
-		if ((* it1).cr_list.size() < nbonds)
+		if ((* it1).GetConnRecCount() < nbonds)
 		{
 			anchor = it1;
-			nbonds = (* anchor).cr_list.size();
+			nbonds = (* anchor).GetConnRecCount();
 		}
 	}
 	
@@ -111,8 +112,8 @@ intcrd::intcrd(model & p1, i32s p2, i32s p3)
 			}
 			
 			i32s search2 = NOT_DEFINED;
-			iter_cl it2 = (* it1).cr_list.begin();
-			while (it2 != (* it1).cr_list.end())
+			iter_cl it2 = (* it1).GetConnRecsBegin();
+			while (it2 != (* it1).GetConnRecsEnd())
 			{
 				search2 = 0;
 				while (search2 < (i32s) added_atoms.size())
@@ -125,7 +126,7 @@ intcrd::intcrd(model & p1, i32s p2, i32s p3)
 				else it2++;
 			}
 			
-			if (it2 != (* it1).cr_list.end())
+			if (it2 != (* it1).GetConnRecsEnd())
 			{
 				added_index = search2;
 				matching_cr = & (* it2);
@@ -145,7 +146,7 @@ intcrd::intcrd(model & p1, i32s p2, i32s p3)
 		
 		if (added_atoms.size() < 2) flag = false;			// skip the first one.
 		
-		if ((* it1).cr_list.size() < 2) flag = false;			// there are further bonds.
+		if ((* it1).GetConnRecCount() < 2) flag = false;			// there are further bonds.
 		
 		if (matching_cr->bndr->bt.GetSymbol1() == 'D') flag = false;	// double bonds are fixed.
 		
