@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "mfinder.h"
 
 #include "v3d.h"
@@ -196,7 +197,7 @@ cout << "nmol = " << mdl->nmol << endl;
 		{
 			for (i32s n3 = 0;n3 < ((i32s) path_vector[n2].size()) - 1;n3++)		// tag the main-chain bonds...
 			{
-				iter_cl it1 = path_vector[n2][n3]->cr_list.begin();
+				iter_cl it1 = path_vector[n2][n3]->GetConnRecsBegin();
 				while ((* it1).atmr != path_vector[n2][n3 + 1]) it1++;
 				(* it1).bndr->flags[0] = true;
 			}
@@ -529,7 +530,7 @@ void mfinder::FindPath(model * mdl, atom * ref1, atom * ref2, i32u index)
 	if (ref1 == ref2) path_vector.push_back(temporary_vector);
 	else
 	{
-		for (iter_cl it1 = ref1->cr_list.begin();it1 != ref1->cr_list.end();it1++)
+		for (iter_cl it1 = ref1->GetConnRecsBegin();it1 != ref1->GetConnRecsEnd();it1++)
 		{
 			if ((* it1).bndr->flags[2]) continue;
 			
@@ -581,7 +582,7 @@ bool mfinder::CheckTemplate(vector<mf_tdata> & tdata, i32s flag)
 	// find all possible bonds that we can use to continue testing...
 	
 	vector<crec> tmpv2; iter_cl it1;
-	for (it1 = tdata[index].ref->cr_list.begin();it1 != tdata[index].ref->cr_list.end();it1++)
+	for (it1 = tdata[index].ref->GetConnRecsBegin();it1 != tdata[index].ref->GetConnRecsEnd();it1++)
 	{
 		bool test1 = (* it1).bndr->flags[flag];
 		if (!test1) tmpv2.push_back((* it1));
