@@ -11,7 +11,6 @@
 #define MODEL_H
 
 #include "libghemicaldefine.h"
-//#include "libghemicalconfig2.h"
 
 class model;
 class crd_set;
@@ -25,10 +24,9 @@ struct readpdb_mdata_chain;
 struct readpdb_data_atom;
 struct readpdb_data_ssbond;
 
-struct ecomp_grp_d;
-
 /*################################################################################################*/
 
+class sequencebuilder;		// seqbuild.h
 class resonance_structures;	// resonance.h
 
 class setup;			// engine.h
@@ -45,7 +43,6 @@ class moldyn_param;		// moldyn.h
 #include "constraint.h"
 
 #include "chn_info.h"
-#include "seqbuild.h"
 
 #include <list>
 #include <vector>
@@ -99,11 +96,23 @@ class model
 	i32s total_traj_frames;		// trajectory files...
 	i32s current_traj_frame;	// trajectory files...
 	
-	vector<const char *> ecomp_grp_name_usr;
+	public:
 	
-//	int ecomp_ngrps;
-//	const char ** ecomp_grp_name;
-//	ecomp_grp_d * ecomp_grp_data;
+	// the verbosity levels control the amount of log messages:
+	// 0 = silent ; 1 = errors ; 2 = warnings ; 3 = notices ; 4 = debug...
+	
+	int verbosity;
+	
+	// the ecomp stuff is here:
+	// ^^^^^^^^^^^^^^^^^^^^^^^^
+	
+	public:
+	
+	bool ecomp_enabled;
+	
+	protected:
+	
+	vector<const char *> ecomp_grp_names;
 	
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -122,14 +131,22 @@ class model
 	public:
 	
 	static const char libversion[16];
+	
+#ifndef WIN32
+	
+	// in windows we need to get the installation path
+	// from system registry... see win32_hacks.cpp
+	
 	static const char libdata_path[256];
 	
+#endif	// WIN32
+	
 	static const char sf_symbols[20 + 1];
 	static const i32s sf_types[SF_NUM_TYPES];
 	static const bool sf_is_polar[SF_NUM_TYPES];
 	
-	static sequencebuilder amino_builder;
-	static sequencebuilder nucleic_builder;
+	static sequencebuilder * amino_builder;
+	static sequencebuilder * nucleic_builder;
 	
 	friend void CopyCRD(model *, engine *, i32u);
 	friend void CopyCRD(engine *, model *, i32u);
@@ -250,17 +267,17 @@ class model
 	iter_al GetAtomsBegin(void) { return atom_list.begin(); }	//const;
 	iter_al GetAtomsEnd(void) { return atom_list.end(); }		//const;
 	atom & GetLastAtom(void) { return atom_list.back(); }
-	i32s GetAtomCount(void) { return atom_list.size(); }	///< This will return atom_list.size().
+	int GetAtomCount(void) { return (int) atom_list.size(); }	///< This will return atom_list.size().
 	
 	iter_bl GetBondsBegin(void) { return bond_list.begin(); }	//const;
 	iter_bl GetBondsEnd(void) { return bond_list.end(); }		//const;
 	bond & GetLastBond(void) { return bond_list.back(); }
-	i32s GetBondCount(void) { return bond_list.size(); }	///< This will return bond_list.size().
+	int GetBondCount(void) { return (int) bond_list.size(); }	///< This will return bond_list.size().
 	
 	iter_CDl GetConstD_begin(void) { return const_D_list.begin(); }		//const;
 	iter_CDl GetConstD_end(void) { return const_D_list.end(); }		//const;
 	constraint_dst & GetLastConstD(void) { return const_D_list.back(); }
-	i32s GetConstD_count(void) { return const_D_list.size(); }	///< This will return const_D_list.size().
+	int GetConstD_count(void) { return (int) const_D_list.size(); }	///< This will return const_D_list.size().
 	
 	// methods for ???
 	// ^^^^^^^^^^^^^^^
@@ -310,12 +327,12 @@ but will not yet permit the use of model::GetRange()-functions since atom list i
 	
 /** This will group the atom list so that molecules (and chains/residues if defined) will form 
 continuous groups. Will permit the use of model::GetRange()-functions. */
-	void SortGroups(bool);
+	void SortGroups(void);
 	
 /** This will validate ref_civ and fill it with chain descriptions using all sequencebuilder objects. 
 Finally it will call model::SortGroups() so that chains and residues in the atom list will be ordered 
 sequentially in contiguous blocks. */
-	virtual void UpdateChains(void);
+	virtual void UpdateChains(bool = false);
 	
 	vector<chn_info> * GetCI(void) { return ref_civ; }
 	
@@ -325,15 +342,17 @@ sequentially in contiguous blocks. */
 This is private because only model::UpdateGroups() should use this... */
 	void GatherAtoms(atom *, i32s);
 	
-	atom * cp_FindAtom(iter_al *, i32s);		///< this is just for CheckProtonation()...
+	atom * cp_FindAtom(iter_al *, i32s);	///< only for CheckProtonation()...
 	
-/** This will create tables of non-standard valences using ref_civ->p_state information. 
+/** This will update atom::formal_charge using ref_civ->p_state information. 
 This is private because only model::AddHydrogens() should use this... */
-	void CheckProtonation(vector<atom *> &, vector<float> &);
+	void CheckProtonation(void);
 	
 	public:
 	
 	void AddHydrogens(void);
+	void AddHydrogens(atom *);
+	
 	void RemoveHydrogens(void);
 	
 	void SolvateBox(fGL, fGL, fGL, fGL = 1.0, model * = NULL, const char * = NULL);
@@ -404,20 +423,10 @@ example to evaluate quality of the data or to match the data with records in oth
 	void EvaluateBFact(void);
 	void EvaluateDiffConst(double);
 	
-// this is not ready yet...
-// ^^^^^^^^^^^^^^^^^^^^^^^^
-	void ecomp_AddGroupU(const char *);
-	bool ecomp_DeleteGroupU(int);
-	
-/*	void ecomp_UnRegisterAll(void);
-	void ecomp_RegisterNoAuto(void);
-	void ecomp_RegisterWithAutoSolv(void);
-	void ecomp_RegisterWithAutoSolvChn1(void);
-	void ecomp_RegisterWithAutoSolvChn2(void);	*/
-	
-	private:
-	
-	void ecomp_Register(void);
+// methods related to E-groups (implemented in eng-classes).
+// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+	int ecomp_AddGroup(const char *);
+	void ecomp_DeleteGroups(void);
 };
 
 /*################################################################################################*/
@@ -520,13 +529,6 @@ struct readpdb_data_ssbond
 	atom * ref;
 };
 
-struct ecomp_grp_d
-{
-	double ECOMP_b;
-	double ECOMP_nbA;
-	double ECOMP_nbB;
-};
-
 /*################################################################################################*/
 
 #endif	// MODEL_H
