@@ -11,7 +11,7 @@
 #define NOTICE_H
 
 #include "libghemicaldefine.h"
-#include "libghemicalconfig2.h"
+#include "libghemical-features.h"
 
 /* added by Robert Williams for Compaq cxx, alpha 11/28/01 */
 #define __USE_STD_IOSTREAM
