@@ -7,15 +7,20 @@
 
 /*################################################################################################*/
 
-#include "geomopt.h"	// required by the FC4 compiler??? strange...
+#include "libghemicalconfig2.h"
 #include "search.h"
 
-#include "geomopt.h"
 #include "utility.h"
 
+#include "eng1_qm_mopac.h"
+
 #include <sstream>
 using namespace std;
 
+#ifdef WIN32
+#include <time.h>
+#endif	// WIN32
+
 /*################################################################################################*/
 
 random_search::random_search(model * p1, i32s p2, i32s p3, i32s p4, i32s p5, i32s p6)
@@ -29,7 +34,7 @@ random_search::random_search(model * p1, i32s p2, i32s p3, i32s p4, i32s p5, i32
 	optsteps = p6;
 	
 	if (!mdl->IsGroupsClean()) mdl->UpdateGroups();		// intcrd requires this!!!
-	if (!mdl->IsGroupsSorted()) mdl->SortGroups(true);	// intcrd requires this!!!
+	if (!mdl->IsGroupsSorted()) mdl->SortGroups();		// intcrd requires this!!!
 	
 	ic = new intcrd((* mdl), molnum, in_crdset);
 	
@@ -140,7 +145,7 @@ systematic_search::systematic_search(model * p1, i32s p2, i32s p3, i32s p4, i32s
 	optsteps = p6;
 	
 	if (!mdl->IsGroupsClean()) mdl->UpdateGroups();		// intcrd requires this!!!
-	if (!mdl->IsGroupsSorted()) mdl->SortGroups(true);	// intcrd requires this!!!
+	if (!mdl->IsGroupsSorted()) mdl->SortGroups();		// intcrd requires this!!!
 	
 	ic = new intcrd((* mdl), molnum, in_crdset);
 	
@@ -271,7 +276,7 @@ monte_carlo_search::monte_carlo_search(model * p1, i32s p2, i32s p3, i32s p4, i3
 // OPTIMIZE WITH NO NONBONDED TERMS TO GET AN IDEAL SYMMETRIC STARTING GEOMETRY?!?!?
 	
 	if (!mdl->IsGroupsClean()) mdl->UpdateGroups();		// intcrd requires this!!!
-	if (!mdl->IsGroupsSorted()) mdl->SortGroups(true);	// intcrd requires this!!!
+	if (!mdl->IsGroupsSorted()) mdl->SortGroups();		// intcrd requires this!!!
 	
 	ic = new intcrd((* mdl), molnum, in_crdset);
 	
@@ -419,20 +424,24 @@ i32s monte_carlo_search::TakeStep(void)
 
 transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 {
-	mdl = p1; deltae = p2; fc[0] = fc[1] = p3;
+	mdl = p1;
+	deltae = p2;
+	fc[0] = fc[1] = p3;
 	
 	init_failed = true;
 	target[0] = target[1] = NULL;
 	
 	setup * su = mdl->GetCurrentSetup();
-	suqm = dynamic_cast<setup1_qm *>(su);
+	suQM = dynamic_cast<setup1_qm *>(su);
+	
+	engQM = NULL;
 	
 	bool bad_setup = false;
-	if (suqm == NULL) bad_setup = true;
+	if (suQM == NULL) bad_setup = true;
 	else
 	{
-		if (suqm->GetCurrEngIndex() < 0) bad_setup = true;	// valid values (MOPAC) are currently from 0 to 3!!!
-		if (suqm->GetCurrEngIndex() > 3) bad_setup = true;	// valid values (MOPAC) are currently from 0 to 3!!!
+		if (suQM->GetCurrEngIndex() < 0) bad_setup = true;	// valid values (MOPAC) are currently from 0 to 3!!!
+		if (suQM->GetCurrEngIndex() > 3) bad_setup = true;	// valid values (MOPAC) are currently from 0 to 3!!!
 	}
 	
 	if (bad_setup)
@@ -474,8 +483,8 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 	
 	if (!identical)
 	{
-		delete[] atab1;
-		delete[] atab2;
+		delete[] atab1; atab1 = NULL;
+		delete[] atab2; atab2 = NULL;
 		
 		mdl->ErrorMessage("No proper pair of reactants/products found!\nPlease see the User's Manual.");
 		return;
@@ -488,6 +497,11 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 	i32s new_csets1 = 2 - mdl->GetCRDSetCount();
 	if (new_csets1 > 0) mdl->PushCRDSets(new_csets1);
 	
+	for (i32s n1 = 0;n1 < mdl->GetCRDSetCount();n1++)
+	{
+		mdl->SetCRDSetVisible(n1, true);	// debug...
+	}
+	
 	for (i32u n1 = 0;n1 < half;n1++)
 	{
 		const fGL * crd = atab2[n1]->GetCRD(0);
@@ -667,8 +681,8 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 		mdl->RemoveAtom(it1);
 	}
 	
-	delete[] atab1;
-	delete[] atab2;
+	delete[] atab1; atab1 = NULL;
+	delete[] atab2; atab2 = NULL;
 	
 	// ...and then superimpose the initial reactant/product structures.
 	
@@ -684,9 +698,9 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 	// ok, the geometry setup is ready.
 	
 	target[0] = new f64[mdl->GetAtomCount() * 3];
-	SetTarget(0, 1);	// use products as target for reactants...
-	
 	target[1] = new f64[mdl->GetAtomCount() * 3];
+	
+	SetTarget(0, 1);	// use products as target for reactants...
 	SetTarget(1, 0);	// use reactants as target for products...
 	
 	// if no patoms were found (that is, no change in bonds; for example amine inversion) set the atom that shifts most!
@@ -725,27 +739,46 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 	
 	// refine the initial structures + other initializations.
 	
-	suqm->DiscardCurrentEngine();	// make sure to release mopac_lock!!!
+	suQM->DiscardCurrentEngine();	// make sure to release mopac_lock!!!
+	
+	// 20061025 TH ; the memory management inside libmopac7 seems to be
+	// more or less messed up ; so far we have created and destroyed the
+	// engine object many times here but since versions 1.9x of ghemical
+	// the transition state search has been VERY unstable. valgrind shows
+	// memory mess-ups inside libmopac7 so here we have a design where
+	// the engine object is created and destroyed only once...
+	
+	engine * eng = suQM->CreateEngineByIndex(suQM->GetCurrEngIndex());
+	engQM = dynamic_cast<eng1_qm *>(eng); eng = NULL;
+	
+	if (engQM == NULL)
+	{
+		cout << "tss : could not create engQM object!" << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	bool problems = false;
+	for (int ii = 0;ii < engQM->GetAtomCount();ii++)
+	{
+		if (engQM->l2g_qm[ii] != ii) problems = true;
+	}
+	
+	if (problems)
+	{
+		cout << "tss : l2g_qm is bad!" << endl;
+		exit(EXIT_FAILURE);
+	}
 	
 	for (i32s n1 = 0;n1 < 2;n1++)
 	{
-		engine * eng1 = suqm->CreateEngineByIndex(suqm->GetCurrEngIndex());
-		eng1_qm * eng2 = dynamic_cast<eng1_qm *>(eng1);
+		CopyCRD(mdl, engQM, n1);
+		engQM->tss_ref_str = target[n1];
+		engQM->tss_force_const = fc[n1];
 		
-		if (eng2 == NULL)
-		{
-			cout << "tss : could not create eng object!" << endl;
-			exit(EXIT_FAILURE);
-		}
+		geomopt * opt = new geomopt(engQM, 20, 0.0125, 10.0);		// optimal settings?!?!?
 		
-		CopyCRD(mdl, eng2, n1);
-		eng2->tss_ref_str = target[n1];
-		eng2->tss_force_const = fc[n1];
-		
-		geomopt * opt = new geomopt(eng2, 20, 0.0125, 10.0);		// optimal settings?!?!?
-		
-		eng2->Compute(0);
-		f64 prev = eng2->energy;
+		engQM->Compute(0);
+		f64 prev = engQM->energy;
 		
 		i32s failed_steps = 0;
 		for (i32s n2 = 0;n2 < 100;n2++)
@@ -762,23 +795,21 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 			if (failed_steps >= 20) break;
 		}
 		
-		delete opt;
+		delete opt; opt = NULL;
 		
 		// store the calculated energy and the optimized structure...
 		
-		eng2->Compute(0);
-		energy1[n1] = eng2->energy;
-		energy2[n1] = eng2->energy - eng2->tss_force_const * eng2->tss_delta_ene;
-		CopyCRD(eng2, mdl, n1);
-		
-		delete eng1;
+		engQM->Compute(0);
+		energy1[n1] = engQM->energy;
+		energy2[n1] = engQM->energy - engQM->tss_force_const * engQM->tss_delta_ene;
+		CopyCRD(engQM, mdl, n1);
 	}
 	
-	SetTarget(0, 1);	// use products as target for reactants...
-	SetTarget(1, 0);	// use reactants as target for products...
+	SetTarget(0, 1);	// update ; use products as target for reactants...
+	SetTarget(1, 0);	// update ; use reactants as target for products...
 	
-	t[0] = energy1[0] + deltae;
-	t[1] = energy1[1] + deltae;
+	t_ene[0] = energy1[0] + deltae;
+	t_ene[1] = energy1[1] + deltae;
 	
 	p[0] = -10000.0;	// set initial reaction coordinates...
 	p[1] = +10000.0;	// ...that are used only to sort the results.
@@ -789,8 +820,31 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 
 transition_state_search::~transition_state_search(void)
 {
-	if (target[0] != NULL) delete[] target[0];
-	if (target[1] != NULL) delete[] target[1];
+	if (target[0] != NULL)
+	{
+		delete[] target[0];
+		target[0] = NULL;
+	}
+	
+	if (target[1] != NULL)
+	{
+		delete[] target[1];
+		target[1] = NULL;
+	}
+	
+	if (engQM != NULL)
+	{
+#ifdef ENABLE_MOPAC7
+		// 20061025 TH ; even the single deletion of the engine object
+		// seems to cause crashes.  :(  so try to just leave it there...
+		eng1_qm_mopac * zzz = dynamic_cast<eng1_qm_mopac *>(engQM);
+		if (zzz != NULL) zzz->FixMeLaterTSS(); else delete engQM;
+#else	// ENABLE_MOPAC7
+		delete engQM;
+#endif	// ENABLE_MOPAC7
+		
+		engQM = NULL;
+	}
 }
 
 void transition_state_search::Run(i32s * prev_not_stored)
@@ -803,20 +857,11 @@ void transition_state_search::Run(i32s * prev_not_stored)
 		
 		// perform constrained optimization...
 		
-		engine * eng1 = suqm->CreateEngineByIndex(suqm->GetCurrEngIndex());
-		eng1_qm * eng2 = dynamic_cast<eng1_qm *>(eng1);
+		CopyCRD(mdl, engQM, n1);
+		engQM->tss_ref_str = target[n1];
+		engQM->tss_force_const = fc[n1];
 		
-		if (eng2 == NULL)
-		{
-			cout << "tss : could not create eng object!" << endl;
-			exit(EXIT_FAILURE);
-		}
-		
-		CopyCRD(mdl, eng2, n1);
-		eng2->tss_ref_str = target[n1];
-		eng2->tss_force_const = fc[n1];
-		
-		geomopt * opt = new geomopt(eng2, 10, 0.0125, 10.0);		// optimal settings?!?!?
+		geomopt * opt = new geomopt(engQM, 10, 0.0125, 10.0);		// optimal settings?!?!?
 		
 		i32s n2 = 0;
 		while (true)
@@ -826,50 +871,41 @@ void transition_state_search::Run(i32s * prev_not_stored)
 			
 			if (n2 != 0 && !(n2 % 20))
 			{
-				if (eng2->tss_delta_ene < 1.0e-15)	// flipped??? test using zero???
+				if (engQM->tss_delta_ene < 1.0e-15)	// flipped??? test using zero???
 				{
-////////////////////////////////////////////////////////////////
-//cout << "FLIPPED!!! n1= " << n1 << endl; int yy;cin>>yy;
-////////////////////////////////////////////////////////////////
-					iter_al it1; i32u counter = 0;
-					for (it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
-					{
-						eng2->crd[counter] = target[!n1][counter++];
-						eng2->crd[counter] = target[!n1][counter++];
-						eng2->crd[counter] = target[!n1][counter++];
-					}
-					
+////////////////////////////////////////////////////////////
+//cout << "FLIPPED!!! n1= " << n1 << endl; int xx;cin>>xx;
+////////////////////////////////////////////////////////////
+					CopyCRD(mdl, engQM, n1);
 					ready[n1] = true;
 					break;
 				}
 				
 				const f64 limit = deltae * 0.10;
-				if (opt->optval > (t[n1] - limit) && opt->optval < (t[n1] + limit)) break;
+				if (opt->optval > (t_ene[n1] - limit) && opt->optval < (t_ene[n1] + limit)) break;
 				
-				f64 diff = t[n1] - opt->optval;
-				eng2->tss_force_const += diff / eng2->tss_delta_ene * 0.50;	// optimal???
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-//cout << "n2 = " << n2 << " E = " << opt->optval << " t = " << t[n1] << " fc = " << eng2->tss_force_const << " de = " << eng2->tss_delta_ene << endl; int qq; cin >> qq;
-//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+				f64 diff = t_ene[n1] - opt->optval;
+				engQM->tss_force_const += diff / engQM->tss_delta_ene * 0.50;	// optimal???
+///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+//cout << "n2 = " << n2 << " E = " << opt->optval << " t_ene = " << t_ene[n1] << " fc = " << engQM->tss_force_const << " de = " << engQM->tss_delta_ene << endl; int yy;cin>>yy;
+///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 			}
 			
 			n2++;
 		}
 		
-		delete opt;
+		delete opt; opt = NULL;
 		
 		if (n1 == 0) p[n1] += 1.0;
 		if (n1 == 1) p[n1] -= 1.0;
 		
 		// copy the optimized structure and update energy...
 		
-		eng2->Compute(0);
-		energy1[n1] = eng2->energy;
-		energy2[n1] = eng2->energy - eng2->tss_force_const * eng2->tss_delta_ene;
-		fc[n1] = eng2->tss_force_const; last_de[n1] = eng2->tss_delta_ene;
-		CopyCRD(eng2, mdl, n1);
-		
-		delete eng1;
+		engQM->Compute(0);
+		energy1[n1] = engQM->energy;
+		energy2[n1] = engQM->energy - engQM->tss_force_const * engQM->tss_delta_ene;
+		fc[n1] = engQM->tss_force_const; last_de[n1] = engQM->tss_delta_ene;
+		CopyCRD(engQM, mdl, n1);
 	}
 }
 
@@ -884,7 +920,7 @@ void transition_state_search::UpdateTargets(bool * update)
 	{
 		if (!update[n1]) continue;
 		
-		t[n1] = energy1[n1] + deltae;
+		t_ene[n1] = energy1[n1] + deltae;
 		
 		SetTarget(!n1, n1);
 		check_fc[!n1] = true;
@@ -895,25 +931,18 @@ void transition_state_search::UpdateTargets(bool * update)
 		if (last_de[n1] < 1.0e-15) continue;	// flipped??? test using zero???
 		if (!check_fc[n1]) continue;
 		
-		engine * eng1 = suqm->CreateEngineByIndex(suqm->GetCurrEngIndex());
-		eng1_qm * eng2 = dynamic_cast<eng1_qm *>(eng1);
+///////////////////////////////////////////////////////
+//cout << "CHECK FC ; " << n1 << endl; int xx;cin>>xx;
+///////////////////////////////////////////////////////
 		
-		if (eng2 == NULL)
-		{
-			cout << "tss : could not create eng object!" << endl;
-			exit(EXIT_FAILURE);
-		}
+		CopyCRD(mdl, engQM, n1);
+		engQM->tss_ref_str = target[n1];
+		engQM->tss_force_const = fc[n1];
 		
-		CopyCRD(mdl, eng2, n1);
-		eng2->tss_ref_str = target[n1];
-		eng2->tss_force_const = fc[n1];
+		engQM->Compute(0);
 		
-		eng2->Compute(0);
-		
-		fc[n1] *= last_de[n1] / eng2->tss_delta_ene;
-		last_de[n1] = eng2->tss_delta_ene;
-		
-		delete eng1;
+		fc[n1] *= last_de[n1] / engQM->tss_delta_ene;
+		last_de[n1] = engQM->tss_delta_ene;
 	}
 }
 
@@ -942,20 +971,20 @@ stationary_state_search::stationary_state_search(engine * p1, i32s p2, f64 p3, f
 	eng = p1;
 	SetNGDelta(1.0e-4);
 	
-	d1 = new f64[eng->GetAtomCount() * 3];
+	my_d1 = new f64[eng->GetAtomCount() * 3];
 	
 	for (i32s n1 = 0;n1 < eng->GetAtomCount();n1++)
 	{
 		for (i32s n2 = 0;n2 < 3;n2++)
 		{
-			AddVar(& eng->crd[n1 * 3 + n2], & d1[n1 * 3 + n2]);
+			AddVar(& eng->crd[n1 * 3 + n2], & my_d1[n1 * 3 + n2]);
 		}
 	}
 }
 
 stationary_state_search::~stationary_state_search(void)
 {
-	delete[] d1;
+	delete[] my_d1;
 }
 
 f64 stationary_state_search::GetValue(void)
