@@ -10,15 +10,14 @@
 #ifndef SEARCH_H
 #define SEARCH_H
 
-//#include "libghemicalconfig2.h"
-
-#define UPDATEFRQ 25
 class random_search;		// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
 class systematic_search;	// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
 class monte_carlo_search;	// DO WE NEED TO RANDOMIZE THE RANDOM NUMBER GENERATOR SOMEWAY???
 
 class transition_state_search;
 
+#define UPDATEFRQ 25
+
 /*################################################################################################*/
 
 #include "engine.h"
@@ -166,7 +165,9 @@ class transition_state_search
 	protected:
 	
 	model * mdl;
-	setup1_qm * suqm;
+	
+	setup1_qm * suQM;
+	eng1_qm * engQM;
 	
 	bool init_failed;
 	
@@ -178,7 +179,7 @@ class transition_state_search
 	
 	f64 fc[2];
 	
-	f64 t[2];		// target
+	f64 t_ene[2];
 	f64 last_de[2];
 	
 	f64 p[2];		// progress ; actually a "reaction coordinate" for sorting the structures.
@@ -222,7 +223,7 @@ class stationary_state_search : public conjugate_gradient
 	
 	engine * eng;
 	
-	f64 * d1;
+	f64 * my_d1;
 	
 	public:
 	
