@@ -11,12 +11,15 @@
 #define SEQBUILD_H
 
 #include "libghemicaldefine.h"
-//#include "libghemicalconfig2.h"
+
+struct sb_constraint_crd;
+struct sb_constraint_tor;
 
 class sb_chain_descriptor;
 
 class sequencebuilder;
 
+class sb_data_td;	// for internal use only...
 class sb_data_atm;	// for internal use only...
 class sb_data_bnd;	// for internal use only...
 
@@ -30,14 +33,30 @@ struct sb_tdata;	// for internal use only...
 #include "bond.h"
 
 #include "chn_info.h"
-
-//#include "model.h"
 #include "typerule.h"
 
 typedef vector<atom *> atmr_vector;
 
 /*################################################################################################*/
 
+struct sb_constraint_crd
+{
+	int pos;
+	
+	int atm_id;
+	float crdX;
+	float crdY;
+	float crdZ;
+};
+
+struct sb_constraint_tor
+{
+	int pos;
+	
+	int tor_ind;
+	float tor_val;
+};
+
 /**	A sb_chain_descriptor object describes the chain to be built for the Build() method of
 	the sequence builder class. It describes the chain using either 1-letter or 3-letter codes
 	and it can optionally contain information about 3D-structure as well.
@@ -52,11 +71,19 @@ class sb_chain_descriptor
 	vector<char> * seq1;
 	vector<char *> * seq3;
 	
+	int curr_res;
+	
+	int c_crd_BGN; int c_crd_END;
+	vector<sb_constraint_crd> c_crd_v;
+	
+	int c_tor_BGN; int c_tor_END;
+	vector<sb_constraint_tor> c_tor_v;
+	
 	friend class sequencebuilder;
 	
 	public:
 	
-	vector<f64> default_torsions;
+	vector<f64> def_tor;
 	
 	public:
 	
@@ -66,8 +93,8 @@ class sb_chain_descriptor
 	int AddRes1(char);		// return value is the current chain length.
 	int AddRes3(const char *);	// return value is the current chain length.
 	
-	
-	
+	void AddCRD(int, float, float, float);
+	void AddTOR(int, float);
 };
 
 /*################################################################################################*/
@@ -91,6 +118,8 @@ class sequencebuilder
 	vector<sb_data_atm> main_vector;
 	vector<sb_data_atm> conn_vector;
 	
+	vector<sb_data_td> td_mc_store;
+	
 	vector<sb_data_res> resi_vector;
 	
 	vector<typerule> head_vector;
@@ -100,11 +129,13 @@ class sequencebuilder
 	
 	char buffer[256];
 	
-	vector<i32s> id_vector;
-	atmr_vector ref_vector;
+	vector<i32s> id_vector;			// the builder
+	atmr_vector atom_vector;		// the builder
+	atmr_vector allatm_vector;		// the builder
+	vector<sb_data_td> td_v;		// the builder
 	
-	atmr_vector temporary_vector;
-	vector<atmr_vector> path_vector;
+	atmr_vector tmpatm_vector;		// the identifier
+	vector<atmr_vector> path_vector;	// the identifier
 	
 	friend class model;
 	friend class setup1_sf;
@@ -114,18 +145,18 @@ class sequencebuilder
 	sequencebuilder(chn_info::chn_type, char *);
 	~sequencebuilder(void);
 	
-	void Build(model *, sb_chain_descriptor *);
+	void Build(model *, sb_chain_descriptor *, bool = true);
 	
 	void Identify(model *);
 	
 	private:
 	
-	// the builder
+// the builder
 	
-	void BuildResidue(model *, sb_data_res *, vector<f64> &);
-	void Convert(atom *[], f64 *, fGL *);
+	void BuildResidue(sb_chain_descriptor *, model *, sb_data_res *);
+	void Convert(sb_chain_descriptor *, sb_data_atm *, fGL *);
 	
-	// the identifier
+// the identifier
 	
 	void BuildTemplate(vector<sb_tdata> &, i32s, bool, bool);
 	void BuildPartialT(vector<sb_tdata> &, vector<sb_data_atm> &);
@@ -135,6 +166,25 @@ class sequencebuilder
 
 /*################################################################################################*/
 
+class sb_data_td
+{
+	protected:
+	
+	i32s id[4];
+	bool flag;
+	
+	friend class sequencebuilder;
+	
+	public:
+	
+	sb_data_td(void);
+	~sb_data_td(void);
+	
+	friend istream & operator>>(istream &, sb_data_td &);
+};
+
+/*################################################################################################*/
+
 class sb_data_atm
 {
 	protected:
@@ -143,6 +193,8 @@ class sb_data_atm
 	i32s prev[3];
 	
 	element el;
+	int f_chrg;
+	
 	bondtype bt;
 	
 	typerule * tr;
@@ -159,7 +211,6 @@ class sb_data_atm
 	~sb_data_atm(void);
 	
 	friend istream & operator>>(istream &, sb_data_atm &);
-	friend ostream & operator<<(ostream &, sb_data_atm &);
 };
 
 /*################################################################################################*/
@@ -194,9 +245,10 @@ class sb_data_res
 	
 	char * description;
 	
+	vector<sb_data_td> td_vector;
 	vector<sb_data_atm> atm_vector;
 	vector<sb_data_bnd> bnd_vector;
-
+	
 	friend class sequencebuilder;
 	
 	friend class model;		// old???
