@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "tab_mm_prmfit.h"
 
 #include "typerule.h"
@@ -25,15 +26,15 @@ prmfit_tables::prmfit_tables(const char * p1)
 	path = new char[strlen(p1) + 1];
 	strcpy(path, p1);
 	
-	ostr = & cout;		// print output.
-//	ostr = NULL;		// do not print output.
-	
 	ifstream file;
 	file.unsetf(ios::dec | ios::oct | ios::hex);
 	
 	ostringstream fns;
 	char buffer[1024];
 	
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
+	
 /*##############################################*/
 /*##############################################*/
 
@@ -220,10 +221,17 @@ void prmfit_tables::PrintAllTypeRules(ostream & p1)
 
 i32u prmfit_tables::UpdateTypes(setup * su)
 {
-	if (ostr != NULL) (* ostr) << "setting up atom types and formal charges..." << endl;
+	model * mdl = su->GetModel();
+	
+	if (mdl->verbosity >= 3)
+	{
+		ostringstream str;
+		str << "Setting up atom types and formal charges..." << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
+	}
 	
 	i32u errors = 0;
-	model * mdl = su->GetModel();
 	
 // determine the atomtypes for all atoms, not just MM atoms (might need to skip the virtual atoms of SF if eng2???).
 // determine the atomtypes for all atoms, not just MM atoms (might need to skip the virtual atoms of SF if eng2???).
@@ -231,8 +239,6 @@ i32u prmfit_tables::UpdateTypes(setup * su)
 	
 	for (iter_al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
 	{
-		if (ostr != NULL && !((* it1).index % 10)) (* ostr) << "*" << flush;
-		
 		i32u at_range[2];
 		
 		at_range[0] = 0;
@@ -269,9 +275,13 @@ i32u prmfit_tables::UpdateTypes(setup * su)
 		}
 		else
 		{
-			ostringstream str1;
-			str1 << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << endl << ends;
-			mdl->PrintToLog(str1.str().c_str());
+			if (mdl->verbosity >= 2)
+			{
+				ostringstream str;
+				str << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << NEWLINE << ends;
+				
+				mdl->PrintToLog(str.str().c_str());
+			}
 			
 			(* it1).atmtp = NOT_DEFINED;
 			(* it1).charge = 0.0;
@@ -281,15 +291,22 @@ i32u prmfit_tables::UpdateTypes(setup * su)
 		}
 	}
 	
-	if (ostr != NULL) (* ostr) << endl;
 	return errors;
 }
 
 i32u prmfit_tables::UpdateCharges(setup * su)
 {
-	if (ostr != NULL) (* ostr) << "setting up partial charges..." << endl;
+	model * mdl = su->GetModel();
+	
+	if (mdl->verbosity >= 3)
+	{
+		ostringstream str;
+		str << "Setting up partial charges..." << NEWLINE << ends;
+		mdl->PrintToLog(str.str().c_str());
+	}
 	
 	i32u errors = 0;
+	
 //	atom ** atmtab = su->GetMMAtoms();
 	bond ** bndtab = su->GetMMBonds();
 	
@@ -300,7 +317,7 @@ i32u prmfit_tables::UpdateCharges(setup * su)
 		query.atmtp[1] = bndtab[n1]->atmr[1]->atmtp;
 		query.bndtp = bndtab[n1]->bt.GetValue();
 		
-		DoParamSearch(& query);
+		DoParamSearch(& query, mdl);
 		if (query.index == NOT_DEFINED) errors++;
 		
 		f64 delta = query.ci;			// here we also determine...
@@ -327,7 +344,7 @@ const prmfit_at * prmfit_tables::GetAtomType(i32s p1)
 	return NULL;
 }
 
-void prmfit_tables::DoParamSearch(prmfit_bs_query * query)
+void prmfit_tables::DoParamSearch(prmfit_bs_query * query, model * mdl)
 {
 	for (i32u n1 = 0;n1 < bs_vector.size();n1++)
 	{
@@ -366,13 +383,16 @@ void prmfit_tables::DoParamSearch(prmfit_bs_query * query)
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 	
-	if (ostr != NULL)
+	if (mdl != NULL && mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown bs: ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
-		(* ostr) << query->bndtp.GetValue() << " ";
-		(* ostr) << endl;
+		ostringstream str;
+		str << "WARNING : unknown bs: ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
+		str << query->bndtp.GetValue() << " ";
+		str << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	// the search failed, return default parameters...
@@ -386,7 +406,7 @@ void prmfit_tables::DoParamSearch(prmfit_bs_query * query)
 	query->ci = 0.0;
 }
 
-void prmfit_tables::DoParamSearch(prmfit_ab_query * query)
+void prmfit_tables::DoParamSearch(prmfit_ab_query * query, model * mdl)
 {
 	for (i32u n1 = 0;n1 < ab_vector.size();n1++)
 	{
@@ -441,17 +461,20 @@ void prmfit_tables::DoParamSearch(prmfit_ab_query * query)
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 	
-	if (ostr != NULL)
+	if (mdl != NULL && mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown ab: " << hex;
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
-		(* ostr) << query->bndtp[0].GetValue() << " ";
-		(* ostr) << query->bndtp[1].GetValue() << " ";
-		(* ostr) << endl;
+		ostringstream str;
+		str << "WARNING : unknown ab: ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
+		str << query->bndtp[0].GetValue() << " ";
+		str << query->bndtp[1].GetValue() << " ";
+		str << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
-
+	
 	// the search failed, return default parameters...
 	
 	query->index = NOT_DEFINED;
@@ -461,7 +484,7 @@ void prmfit_tables::DoParamSearch(prmfit_ab_query * query)
 	query->fc = 250.0;
 }
 
-void prmfit_tables::DoParamSearch(prmfit_tr_query * query)
+void prmfit_tables::DoParamSearch(prmfit_tr_query * query, model * mdl)
 {
 	for (i32u n1 = 0;n1 < tr_vector.size();n1++)
 	{
@@ -521,17 +544,20 @@ void prmfit_tables::DoParamSearch(prmfit_tr_query * query)
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 	
-	if (ostr != NULL)
+	if (mdl != NULL && mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown tr: " << hex;
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[3] << dec << " ";
-		(* ostr) << query->bndtp[0].GetValue() << " ";
-		(* ostr) << query->bndtp[1].GetValue() << " ";
-		(* ostr) << query->bndtp[2].GetValue() << " ";
-		(* ostr) << endl;
+		ostringstream str;
+		str << "WARNING : unknown tr: ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[3] << dec << " ";
+		str << query->bndtp[0].GetValue() << " ";
+		str << query->bndtp[1].GetValue() << " ";
+		str << query->bndtp[2].GetValue() << " ";
+		str << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	// the search failed, return default parameters...
@@ -544,7 +570,7 @@ void prmfit_tables::DoParamSearch(prmfit_tr_query * query)
 	query->fc3 = 0.0;
 }
 
-void prmfit_tables::DoParamSearch(prmfit_op_query * query)
+void prmfit_tables::DoParamSearch(prmfit_op_query * query, model * mdl)
 {
 	for (i32u n1 = 0;n1 < op_vector.size();n1++)
 	{
@@ -615,17 +641,20 @@ void prmfit_tables::DoParamSearch(prmfit_op_query * query)
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 // recursive search?!?!?! DO IT BY CALLING AGAIN USING SECONDARY TYPES!!! NOT RECURSIVELY!!!
 	
-	if (ostr != NULL)
+	if (mdl != NULL && mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown op: " << hex;
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << query->atmtp[3] << dec << " ";
-		(* ostr) << query->bndtp[0].GetValue() << " ";
-		(* ostr) << query->bndtp[1].GetValue() << " ";
-		(* ostr) << query->bndtp[2].GetValue() << " ";
-		(* ostr) << endl;
+		ostringstream str;
+		str << "WARNING : unknown op: ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[1] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[2] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << query->atmtp[3] << dec << " ";
+		str << query->bndtp[0].GetValue() << " ";
+		str << query->bndtp[1].GetValue() << " ";
+		str << query->bndtp[2].GetValue() << " ";
+		str << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	// the search failed, return default parameters...
