@@ -10,8 +10,6 @@
 #ifndef TAB_MM_PRMFIT_H
 #define TAB_MM_PRMFIT_H
 
-//#include "libghemicalconfig2.h"
-
 struct prmfit_at;		// atomtype
 
 struct prmfit_bs;		// bond stretching
@@ -28,6 +26,8 @@ class prmfit_tables;
 
 /*################################################################################################*/
 
+class typerule;		// typerule.h
+
 #include "eng1_mm_prmfit.h"
 #include "conjgrad.h"
 
@@ -215,10 +215,6 @@ class prmfit_tables
 	vector<prmfit_ab> ab_vector;
 	vector<prmfit_tr> tr_vector;
 	vector<prmfit_op> op_vector;
-
-	public:
-	
-	ostream * ostr;		///< an optional output stream, defaults to (& cout); may be set to NULL.
 	
 	public:
 	
@@ -235,10 +231,10 @@ class prmfit_tables
 	
 	const prmfit_at * GetAtomType(i32s);
 	
-	void DoParamSearch(prmfit_bs_query *);
-	void DoParamSearch(prmfit_ab_query *);
-	void DoParamSearch(prmfit_tr_query *);
-	void DoParamSearch(prmfit_op_query *);
+	void DoParamSearch(prmfit_bs_query *, model *);
+	void DoParamSearch(prmfit_ab_query *, model *);
+	void DoParamSearch(prmfit_tr_query *, model *);
+	void DoParamSearch(prmfit_op_query *, model *);
 };
 
 /*################################################################################################*/
