@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "tab_mm_tripos52.h"
 
 #include "typerule.h"
@@ -24,16 +25,15 @@ singleton_cleaner<tripos52_tables> tripos52_cleaner(tripos52_tables::GetInstance
 
 tripos52_tables::tripos52_tables(void)
 {
-	ostr = NULL;		// do not print output.
-//	ostr = & cout;		// print output.
-	
-	const char * fn;
-	
 	ifstream file;
 	file.unsetf(ios::dec | ios::oct | ios::hex);
 	
+	const char * fn;
 	char buffer[1024];
 	
+	ostream * ostr = NULL;		// do not print output.
+//	ostream * ostr = & cout;	// print output to cout.
+	
 /*##############################################*/
 /*##############################################*/
 	
@@ -230,15 +230,20 @@ void tripos52_tables::PrintAllTypeRules(ostream & p1)
 
 i32u tripos52_tables::UpdateTypes(setup * su)
 {
-	if (ostr != NULL) (* ostr) << "setting up atom types and formal charges..." << endl;
+	model * mdl = su->GetModel();
+	
+	if (mdl->verbosity >= 3)
+	{
+		ostringstream str;
+		str << "Setting up atom types and formal charges..." << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
+	}
 	
 	i32u errors = 0;
-	model * mdl = su->GetModel();
 	
 	for (iter_al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
 	{
-		if (ostr != NULL && !((* it1).index % 10)) (* ostr) << "*" << flush;
-		
 		i32u at_range[2];
 		
 		at_range[0] = 0;
@@ -277,9 +282,13 @@ i32u tripos52_tables::UpdateTypes(setup * su)
 		}
 		else
 		{
-			ostringstream str1;
-			str1 << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << endl << ends;
-			mdl->PrintToLog(str1.str().c_str());
+			if (mdl->verbosity >= 2)
+			{
+				ostringstream str;
+				str << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << NEWLINE << ends;
+				
+				mdl->PrintToLog(str.str().c_str());
+			}
 			
 			(* it1).atmtp = NOT_DEFINED;
 			(* it1).charge = 0.0;
@@ -289,20 +298,25 @@ i32u tripos52_tables::UpdateTypes(setup * su)
 		}
 	}
 	
-	if (ostr != NULL) (* ostr) << endl;
 	return errors;
 }
 
 i32u tripos52_tables::UpdateCharges(setup * su)
 {
-	if (ostr != NULL) (* ostr) << "setting up partial charges..." << endl;
+	model * mdl = su->GetModel();
+	
+	if (mdl->verbosity >= 3)
+	{
+		ostringstream str;
+		str << "Setting up partial charges..." << NEWLINE << ends;
+		mdl->PrintToLog(str.str().c_str());
+	}
 	
 	i32u errors = 0;
-	model * mdl = su->GetModel();
 	
 	for (iter_bl it1 = mdl->GetBondsBegin();it1 != mdl->GetBondsEnd();it1++)
 	{
-		f64 delta = tripos52_tables::GetInstance()->GetChargeInc(& (* it1));
+		f64 delta = tripos52_tables::GetInstance()->GetChargeInc(& (* it1), mdl);
 		
 		(* it1).atmr[0]->charge -= delta;
 		(* it1).atmr[1]->charge += delta;
@@ -311,7 +325,7 @@ i32u tripos52_tables::UpdateCharges(setup * su)
 	return errors;
 }
 
-f64 tripos52_tables::GetChargeInc(bond * ref)
+f64 tripos52_tables::GetChargeInc(bond * ref, model * mdl)
 {
 	i32s atmtp[2];
 	for (i32s n1 = 0;n1 < 2;n1++)
@@ -336,12 +350,15 @@ f64 tripos52_tables::GetChargeInc(bond * ref)
 		}
 	}
 	
-	if (ostr != NULL)
+	if (mdl != NULL && mdl->verbosity >= 2)
 	{
-		(* ostr) << "there was no record for the following ci: ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
-		(* ostr) << ref->bt.GetValue() << endl;
+		ostringstream str;
+		str << "WARNING : there was no record for the following ci: ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
+		str << ref->bt.GetValue() << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	return 0.0;
@@ -390,12 +407,16 @@ bool tripos52_tables::Init(eng1_mm * eng, mm_tripos52_bt1 * ref, i32s bt)
 		}
 	}
 	
-	if (ostr != NULL)
+	model * mdl = eng->GetSetup()->GetModel();
+	if (mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown bst: ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
-		(* ostr) << bt << endl;
+		ostringstream str;
+		str << "WARNING : unknown bst: ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
+		str << bt << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	ref->opt = 0.1 * 1.100;
@@ -456,13 +477,17 @@ bool tripos52_tables::Init(eng1_mm * eng, mm_tripos52_bt2 * ref, i32s * bt)
 		}
 	}
 	
-	if (ostr != NULL)
+	model * mdl = eng->GetSetup()->GetModel();
+	if (mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown abn: " << hex;
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
-		(* ostr) << bt[0] << " " << bt[1] << endl;
+		ostringstream str;
+		str << "WARNING : unknown abn: " << hex;
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
+		str << bt[0] << " " << bt[1] << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	ref->opt = M_PI * 120.0 / 180.0;
@@ -527,14 +552,18 @@ bool tripos52_tables::Init(eng1_mm * eng, mm_tripos52_bt3 * ref, i32s * bt)
 		}
 	}
 	
-	if (ostr != NULL)
+	model * mdl = eng->GetSetup()->GetModel();
+	if (mdl->verbosity >= 2)
 	{
-		(* ostr) << "unknown tor: " << hex;
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
-		(* ostr) << "0x" << hex << setw(4) << setfill('0') << atmtp[3] << dec << " ";
-		(* ostr) << bt[0] << " " << bt[1] << " " << bt[2] << endl;
+		ostringstream str;
+		str << "WARNING : unknown tor: " << hex;
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[0] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[1] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[2] << dec << " ";
+		str << "0x" << hex << setw(4) << setfill('0') << atmtp[3] << dec << " ";
+		str << bt[0] << " " << bt[1] << " " << bt[2] << NEWLINE << ends;
+		
+		mdl->PrintToLog(str.str().c_str());
 	}
 	
 	ref->k = 0.0;
@@ -566,7 +595,15 @@ bool tripos52_tables::Init(eng1_mm * eng, mm_tripos52_nbt1 * ref, bool is14)
 		
 		if (index[n1] == lj_vector.size())
 		{
-			if (ostr != NULL) (* ostr) << "bad atomtype!!! using hydrogen instead..." << endl;
+			model * mdl = eng->GetSetup()->GetModel();
+			if (mdl->verbosity >= 2)
+			{
+				ostringstream str;
+				str << "WARNING : bad atomtype ; using hydrogen instead..." << NEWLINE << ends;
+				
+				mdl->PrintToLog(str.str().c_str());
+			}
+			
 			index[n1] = 0;
 		}
 	}
