@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "typerule.h"
 
 #include <string.h>
@@ -237,11 +238,17 @@ i32s typerule::ReadSubRule(istream * istr, ostream * ostr)
 
 bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 {
+	if ((flag + 1) >= BOND_NFLAGS)
+	{
+		cout << "callEXIT : flag array overflow at typerule::CheckRules()." << endl;
+		exit(EXIT_FAILURE);
+	}
+	
 	i32s n1; iter_cl it1;
 	switch (sr_vector[rule].type)
 	{
 		case tr_subrule::BondedTo:	n1 = false;	// the result...
-		for (it1 = atom->cr_list.begin();it1 != atom->cr_list.end();it1++)
+		for (it1 = atom->GetConnRecsBegin();it1 != atom->GetConnRecsEnd();it1++)
 		{
 			if ((* it1).bndr->flags[flag]) continue;
 			if (sr_vector[rule].el.GetAtomicNumber() != NOT_DEFINED && sr_vector[rule].el.GetAtomicNumber() != (* it1).atmr->el.GetAtomicNumber()) continue;
@@ -259,8 +266,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		return false;
 		
 		case tr_subrule::Valence:			// count the valences as electrons ; easier to use int!!!
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			switch ((* it1++).bndr->bt.GetValue())
 			{
@@ -280,13 +287,13 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NeighborsAll:
-		if (atom->cr_list.size() != (i32u) sr_vector[rule].data) return false;
+		if (atom->GetConnRecCount() != (i32u) sr_vector[rule].data) return false;
 		else if (sr_vector[rule].next != NOT_DEFINED) return CheckRules(mdl, atom, flag, sr_vector[rule].next);
 		else return true;
 		
 		case tr_subrule::NeighborsHeavy:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).atmr->el.GetAtomicNumber() != 1) n1++;
 		}
@@ -295,8 +302,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NeighborsHydrogen:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).atmr->el.GetAtomicNumber() == 1) n1++;
 		}
@@ -305,8 +312,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NeighborsHighElNeg:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			switch ((* it1++).atmr->el.GetAtomicNumber())
 			{
@@ -318,14 +325,14 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 				case 85:	n1++; break;	// At
 			}
 		}
-//		if (n1 != sr_vector[rule].data) return false;
+//	if (n1 != sr_vector[rule].data) return false;
 		if (n1 < sr_vector[rule].data) return false;	// "inexact" test ; use "<" instead of "!="...
 		else if (sr_vector[rule].next != NOT_DEFINED) return CheckRules(mdl, atom, flag, sr_vector[rule].next);
 		else return true;
 		
 		case tr_subrule::NumBondsSingle:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).bndr->bt.GetValue() == BONDTYPE_SINGLE) n1++;
 		}
@@ -334,8 +341,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NumBondsCnjgtd:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).bndr->bt.GetValue() == BONDTYPE_CNJGTD) n1++;
 		}
@@ -344,8 +351,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NumBondsDouble:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).bndr->bt.GetValue() == BONDTYPE_DOUBLE) n1++;
 		}
@@ -354,8 +361,8 @@ bool typerule::CheckRules(model * mdl, atom * atom, i32s flag, i32s rule)
 		else return true;
 		
 		case tr_subrule::NumBondsTriple:
-		n1 = 0; it1 = atom->cr_list.begin();
-		while (it1 != atom->cr_list.end())
+		n1 = 0; it1 = atom->GetConnRecsBegin();
+		while (it1 != atom->GetConnRecsEnd())
 		{
 			if ((* it1++).bndr->bt.GetValue() == BONDTYPE_TRIPLE) n1++;
 		}
