@@ -7,6 +7,7 @@
 
 /*################################################################################################*/
 
+#include "libghemicalconfig2.h"
 #include "utility.h"
 
 #include "v3d.h"
@@ -377,8 +378,12 @@ void DefineSecondaryStructure(model * mdl)
 	
 	for (i32s n1 = 0;n1 < ((i32s) ci_vector.size()) - 1;n1++)
 	{
+		if (ci_vector[n1].type != chn_info::amino_acid) continue;
+		
 		for (i32s n2 = n1 + 1;n2 < (i32s) ci_vector.size();n2++)
 		{
+			if (ci_vector[n2].type != chn_info::amino_acid) continue;
+			
 			cout << "." << flush;
 			
 			for (i32s n3 = 1;n3 < ci_vector[n1].length - 1;n3++)
@@ -429,6 +434,8 @@ void DefineSecondaryStructure(model * mdl)
 	
 	for (i32u n1 = 0;n1 < ci_vector.size();n1++)
 	{
+		if (ci_vector[n1].type != chn_info::amino_acid) continue;
+		
 		cout << "// chain " << n1 << ":" << endl;
 		
 		i32u length = strlen(ci_vector[n1].ss_state);
@@ -493,8 +500,8 @@ f64 HBondEnergy(model * mdl, i32s * res1, i32s * res2)
 	if (it_c == r1b[1]) return 0.0;
 	
 	atom * ref_h = NULL;
-	list<crec>::iterator it1 = (* it_n).cr_list.begin();
-/*	while (it1 != (* it_n).cr_list.end() && ref_h == NULL)
+	list<crec>::iterator it1 = (* it_n).GetConnRecsBegin();
+/*	while (it1 != (* it_n).GetConnRecsEnd() && ref_h == NULL)
 	{
 		if ((* it1).atmr->el.GetAtomicNumber() != 1) it1++;
 		else ref_h = (* it1).atmr;
@@ -506,8 +513,8 @@ f64 HBondEnergy(model * mdl, i32s * res1, i32s * res2)
 	// therefore we don't use the real hydrogens here at all, but we "reconstruct"
 	// the amide hydrogen using the main-chain heavy atoms...
 	
-it1 = (* it_n).cr_list.end();	// pretend that we couldn't find the amide hydrogen...
-	if (it1 == (* it_n).cr_list.end())
+it1 = (* it_n).GetConnRecsEnd();	// pretend that we couldn't find the amide hydrogen...
+	if (it1 == (* it_n).GetConnRecsEnd())
 	{
 		if (!res2[1]) return 0.0;	// skip if N-terminal residue...
 		iter_al r2c[2]; mdl->GetRange(2, r2a, res2[1] - 1, r2c);
@@ -545,7 +552,7 @@ it1 = (* it_n).cr_list.end();	// pretend that we couldn't find the amide hydroge
 	v3d<fGL> oh_v = v3d<fGL>((* it_o).GetCRD(0), ref_h->GetCRD(0));
 	v3d<fGL> cn_v = v3d<fGL>((* it_c).GetCRD(0), (* it_n).GetCRD(0));
 	
-	if (it1 == (* it_n).cr_list.end()) delete ref_h;
+	if (it1 == (* it_n).GetConnRecsEnd()) delete ref_h;
 	
 	f64 tmp1 = 1.0 / on_v.len() + 1.0 / ch_v.len();
 	f64 tmp2 = 1.0 / oh_v.len() + 1.0 / cn_v.len();
