@@ -10,12 +10,8 @@
 #ifndef V3D_H
 #define V3D_H
 
-//#include "libghemicalconfig2.h"
-
 /*################################################################################################*/
 
-#include "typedef.h"
-
 #include <math.h>
 
 #include <iostream>
@@ -54,12 +50,12 @@ template <class TYPE1> class v3d
 	
 	v3d(const TYPE1 * p1)
 	{
-		for (i32s n1 = 0;n1 < 3;n1++) data[n1] = p1[n1];
+		for (int n1 = 0;n1 < 3;n1++) data[n1] = p1[n1];
 	}
 	
 	v3d(const TYPE1 * p1, const TYPE1 * p2)
 	{
-		for (i32s n1 = 0;n1 < 3;n1++) data[n1] = p2[n1] - p1[n1];
+		for (int n1 = 0;n1 < 3;n1++) data[n1] = p2[n1] - p1[n1];
 	}
 	
 	v3d(TYPE1 p1, TYPE1 p2, TYPE1 p3)
@@ -74,7 +70,7 @@ template <class TYPE1> class v3d
 	TYPE1 len(void) const
 	{
 		TYPE1 sum = 0.0;
-		for (i32s n1 = 0;n1 < 3;n1++) sum += data[n1] * data[n1];
+		for (int n1 = 0;n1 < 3;n1++) sum += data[n1] * data[n1];
 		return sqrt(sum);
 	}
 	
@@ -117,7 +113,7 @@ template <class TYPE1> class v3d
 	TYPE1 spr(const v3d<TYPE1> & v1) const
 	{
 		TYPE1 sum = 0.0;
-		for (i32s n1 = 0;n1 < 3;n1++) sum += data[n1] * v1.data[n1];
+		for (int n1 = 0;n1 < 3;n1++) sum += data[n1] * v1.data[n1];
 		return sum;
 	}
 	
@@ -133,32 +129,32 @@ template <class TYPE1> class v3d
 	v3d<TYPE1> operator+(const v3d<TYPE1> & v1) const
 	{
 		v3d<TYPE1> v2;
-		for (i32s n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] + v1.data[n1];
+		for (int n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] + v1.data[n1];
 		return v2;
 	}
 	
 	v3d<TYPE1> operator-(const v3d<TYPE1> & v1) const
 	{
 		v3d<TYPE1> v2;
-		for (i32s n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] - v1.data[n1];
+		for (int n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] - v1.data[n1];
 		return v2;
 	}
 	
 	v3d<TYPE1> operator*(TYPE1 p1) const
 	{
 		v3d<TYPE1> v2;
-		for (i32s n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] * p1;
+		for (int n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] * p1;
 		return v2;
 	}
 	
 	v3d<TYPE1> operator/(TYPE1 p1) const
 	{
 		v3d<TYPE1> v2;
-		for (i32s n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] / p1;
+		for (int n1 = 0;n1 < 3;n1++) v2.data[n1] = data[n1] / p1;
 		return v2;
 	}
 	
-	TYPE1 & operator[](i32s p1) const
+	TYPE1 & operator[](int p1) const
 	{
 		return (TYPE1 &) data[p1];
 	}
